;;;
;;; io.h
;;;
;;; created: <Fri May  7 21:07:42 1999>
;;; last modified: <Thu Jun  1 17:24:55 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This defines the macros for calling the I/O functions in libASM.a
;;;

%ifndef _io_h_
%define _io_h_

;;;*
;;;* WriteString
;;;*
;;;*  In: WriteString fd, *buf (fd [eax+0], *buf[eax+4])
;;;*  Out: eax=0(on success); eax=-1(on error)
;;;*
;;;*  Write *buf to fd. buf is ASCIIZ
;;;*
%macro WriteString 1-2
	extern _WriteString
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _WriteString
%if %0 = 2
	add esp,byte 8
%endif
%endm

;;;*
;;;* WriteStr
;;;*
;;;*  In: WriteStr 'String'
;;;*  Out: eax=0(on success); eax=-1(on error)
;;;*
;;;*  Write String to STDOUT.
;;;*
%macro WriteStr 1+
	section .data
%%data:	db %1,0
	section .text
	WriteString dword STDOUT, dword %%data
%endm

;;;*
;;;* WriteErr
;;;*
;;;*  In: WriteErr 'String'
;;;*  Out: eax=0(on success); eax=-1(on error)
;;;*
;;;*  Write String to STDERR.
;;;*
%macro WriteErr 1+
	section .data
%%data:	db %1,0
	section .text
	WriteString dword STDERR, dword %%data
%endm


;;;*
;;;* WriteChar
;;;*
;;;*  In: WriteChar fd, dword (byte char) (fd [eax+0], char [eax+4])
;;;*  Out: eax=0 (on success); eax=-1 (on error)
;;;*
;;;*  Write Char to fd
;;;*
%macro WriteChar 1-2
	extern _WriteChar
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push dword %2, %1
	mov eax,esp
%endif
	call _WriteChar
%if %0 = 2
	add esp,byte 8
%endif
%endm

;;;*
;;;* WriteChr
;;;*
;;;*  In: WriteChr 'char'
;;;*  Out: eax=0 (on success); eax=-1 (on error)
;;;*
;;;*  Write Char to STDOUT
;;;*
%macro WriteChr 1
	section .text
	WriteChar dword STDOUT, %1
%endm


;;;*
;;;* GetChar
;;;*
;;;*  In: GetChar fd (eax)
;;;*  Out: eax=Char (on success); eax=-1 (on error)
;;;*
;;;*  Read Char from fd
;;;*
%macro GetChar 1
	extern _GetChar
%ifnidni %1,eax
	mov eax,%1
%endif
	call _GetChar
%endm

;;;*
;;;* GetChr
;;;*
;;;*  In: GetChr
;;;*  Out: eax=Char (on success); eax=-1 (on error)
;;;*
;;;*  Read Char from STDIN
;;;*
%macro GetChr 0
	section .text
	GetChar dword STDIN
%endm

;;;*
;;;* GetString
;;;*
;;;*  In: GetString fd, *buf, n (fd [eax+0], *buf [eax+4], n [eax+8])
;;;*  Out: eax=Bytes read (on success); eax=-1 (on error)
;;;*
;;;*  Read string from fd into *buf, length n.
;;;*
%macro GetString 1-3
	extern _GetString
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
 %error "Macro `GetString' expects 1 or 3 parameters."
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _GetString
%if %0 = 3
	add esp,byte 12
%endif
%endm

;;;*
;;;* GetStr
;;;*
;;;*  In: GetStr *buf, n
;;;*  Out: eax=Bytes read (on success); eax=-1 (on error)
;;;*
;;;*  Read string from STDIN into *buf, length n.
;;;*
%macro GetStr 2
	section .text
	GetString dword STDIN, %1, %2
%endm

;;;*
;;;* Writef
;;;*
;;;*  In: Writef fd, *buf, *args (fd [eax+0], *buf [eax+4], *args [eax+8])
;;;*      buf: db "TEXT",0
;;;*      args: dd 1. param, 2.param
;;;*  Out: eax=0 (on success); eax=-1 (on error)
;;;*
;;;*  Writes buf to fd. buf is ASCIIZ.
;;;*  '%%' = '%', '%a'=INT, '%b'=8 bit HEX, '%c'=16 bit HEX, '%d'=32 bit HEX
;;;*  '%e'=*char, '%f'=signed INT, '%gxx' Float with precision xx, '%hxx' Float
;;;*  with precision xx scientific output (xEy = x*10^y), '%ixx' Float with
;;;*  precision xx scientific output (xey = x*e^y)
;;;*  '\n'=lf, '\\'='\', '\t'=ht, '\r'=cr, '\a'=bel
;;;*  '\b'=bs, '\0'=null, '\v'=vt, '\f'=ff
;;;*

%macro Writef 1-3
	extern _Writef
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
 %error "Macro `Writef' expects 1 or 3 parameters."
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _Writef
%if %0 = 3
	add esp,byte 12
%endif
%endm

;;; old:
;;; %macro WritefANY 2-*
;;; 	section .data
;;; %%data:	db %2,0
;;; 	section .text
;;; %if %0 > 2
;;; %rep %0-2
;;; %rotate -1
;;; 	push dword %1
;;; %endrep
;;; %rotate -2
;;; 	push esp
;;; 	push dword %%data
;;; 	push dword %1
;;; 	mov eax,esp
;;; 	Writef eax
;;; 	add esp,((%0-2)*4)+12
;;; %else
;;; 	Writef dword %1, dword %%data, dword 0
;;; %endif
;;; %endm

;;; Improved!
;;;*
;;;* WritefANY
;;;*
;;;*  In: WritefANY fd, 'String', 'String2', param1, param2, ...
;;;*  Out: eax=0 (on success); eax=-1 (on error)
;;;*
;;;*  Writes String to fd. Calls Writef, so args are the same.
;;;*  StringX can be a byte number too (without 'byte'-prefix!). ParamX
;;;*  is a register, a label, an adress or a number (identifier, with
;;;'  'dword'-prefix).
;;;*

%define nul 0
%define bel 7
%define bs 8
%define ht 9
%define lf 10
%define vt 11
%define ff 12
%define cr 13

%macro WritefANY 2-*
%assign tmpcount 0
%undef tmpflag

;;; Find out, how many args are to be interpreted as STRING and store them
	section .data
%%data:	
%rotate 1
%rep %0-1
 %ifndef tmpflag
  %ifnid %1
   %assign tmpcount tmpcount+1
	db %1
  %else
   %define tmpflag
  %endif
 %endif
%rotate 1
%endrep
	db 0

;;; Process parameters
	section .text
%if %0 > (tmpcount+1)
%rep %0-(tmpcount+1)
%rotate -1
	push %1
%endrep
%rotate -(tmpcount+1)
	push esp
	push dword %%data
	push %1
	mov eax,esp
	Writef eax
	add esp,((%0-(tmpcount+1))*4)+12
%else
	Writef %1, dword %%data, dword 0
%endif
%endm

;;;*
;;;* WritefSTDOUT
;;;*
;;;*  In: WritefSTDOUT 'String', param1, param2, ...
;;;*  Out: eax=0 (on success); eax=-1 (on error)
;;;*
;;;*  Writes String to STDOUT. Calls WritefANY, so args are the same.
;;;*
%macro WritefSTDOUT 1+
	WritefANY dword STDOUT, %1
%endm

;;;*
;;;* WritefSTDERR
;;;*
;;;*  In: WritefSTDERR 'String', param1, param2, ...
;;;*  Out: eax=0 (on success); eax=-1 (on error)
;;;*
;;;*  Writes String to STDERR. Calls WritefANY, so args are the same.
;;;*
%macro WritefSTDERR 1+
	WritefANY dword STDERR, %1
%endm

;;;*
;;;* WriteDebugInfo
;;;*
;;;*  In: WriteDebugInfo fd, 'String', param1, param2, ...
;;;*  Out: eax=0 (on success); eax=-1 (on error)
;;;*
;;;*  Writes String (FILENAME:LINE:String) to fd. Calls WritefANY, so args
;;;*  are the same.
;;;*
%macro WriteDebugInfo 2+
%ifndef WriteDebugInfo_FILE_DEFINED
%define WriteDebugInfo_FILE_DEFINED
	section .data
WriteDebugInfo_FILE: db __FILE__,0
%endif
	section .text
	WritefANY %1, "%e:%a:", dword WriteDebugInfo_FILE, dword __LINE__
	WritefANY %1, %2
%endm


;;;*
;;;* WriteInt
;;;*
;;;*  In: WriteInt fd, reg (fd [eax+0], reg [eax+4])
;;;*  Out: -
;;;*
;;;*  Convert reg to ASCII-Int and write to FD
;;;*
%macro WriteInt 1-2
	extern _WriteInt
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _WriteInt
%if %0 = 2
	add esp,byte 8
%endif
%endm


;;;*
;;;* WriteHex
;;;*
;;;*  In: WriteHex fd, reg, width (8, 16, 32) (fd [eax+0], reg [eax+4], width
;;;*      [eax+8])
;;;*  Out: -
;;;*
;;;*  Convert reg (width) to ASCII-Hex and write to STDOUT/FD
;;;*
%macro WriteHex 1-3
	extern _WriteHex
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1, dword STDOUT
	mov eax,esp
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _WriteHex
%if %0 != 1
	add esp,byte 12
%endif
%endm


;;;*
;;;* WriteFloat
;;;*
;;;*  In: WriteFloat fd, *reg, prec (fd [eax+0], *reg [eax+4], prec [eax+8])
;;;*  Out: -
;;;*
;;;*  Convert *reg (qword float) to ASCII and write to STDOUT/FD
;;;*
%macro WriteFloat 1-3
	extern _WriteFloat
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1, dword STDOUT
	mov eax,esp
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _WriteFloat
%if %0 != 1
	add esp,byte 12
%endif
%endm


;;;*
;;;* WriteFloatScE
;;;*
;;;*  In: WriteFloatScE fd, *reg, prec (fd [eax+0], *reg [eax+4], prec [eax+8])
;;;*  Out: -
;;;*
;;;*  Convert *reg (qword float) to ASCII and write to STDOUT/FD
;;;*  in scientific format (xEy = x*10^y)
;;;*
%macro WriteFloatScE 1-3
	extern _WriteFloatScE
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1, dword STDOUT
	mov eax,esp
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _WriteFloatScE
%if %0 != 1
	add esp,byte 12
%endif
%endm


;;;*
;;;* WriteFloatSce
;;;*
;;;*  In: WriteFloatSce fd, *reg, prec (fd [eax+0], *reg [eax+4], prec [eax+8])
;;;*  Out: -
;;;*
;;;*  Convert *reg (qword float) to ASCII and write to STDOUT/FD
;;;*  in scientific format (xey = x*e^y)
;;;*
%macro WriteFloatSce 1-3
	extern _WriteFloatSce
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1, dword STDOUT
	mov eax,esp
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _WriteFloatSce
%if %0 != 1
	add esp,byte 12
%endif
%endm


;;;*
;;;* WaitKeypressed
;;;*
;;;*  In: WaitKeypressed
;;;*  Out: eax=Char (on success); eax=-1 (on error)
;;;*
;;;*  Wait for key pressed and read it out.
;;;*
%macro WaitKeypressed 0
	extern _WaitKeypressed
	call _WaitKeypressed
%endm


%endif	; _io_h_



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: