;;;
;;; seq.h
;;;
;;; created: <Mon Sep 13 13:23:40 1999>
;;; last modified: <Tue May  9 18:43:03 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

;;;
;;; General SEQ commands
;;;

%ifndef _seq_h_
%define _seq_h_

%define SEQ_buf_size 2048

;;;*
;;;* The struc snd is returned by SEQ_open. It us used to store some control
;;;* values that are used by libASM.
;;;*
;;;*  .fd:			fd for opened SEQ
;;;*  .memavail:		Memory availible on SEQ
;;;*  .seqbufptr:		Pointer in .seqbuf
;;;*  .device:		Device number
;;;*  .synth_type:		Synthesiser type
;;;*  .synth_subtype:	Synthesiser subtype
;;;*  .perc_mode:
;;;*  .nr_voices:
;;;*  .nr_drums:
;;;*  .instr_bank_size:
;;;*  .capabilities:
;;;*  .dummies:		Reserved space
;;;*  .name:		SEQ name
;;;*  .seqbuf:		Size of buffer refered in .seqbufptr
;;;*  .seqdev_name:		Devicename ASCIIZ
;;;*

;;; This struc is returned by SEQ_open.
struc snd
.fd:		resd 1			; fd for opened SEQ
.memavail:	resd 1			; Memory availible on SEQ
.seqbufptr:	resd 1			; Pointer in .seqbuf
.device:	resd 1			; Device number
.synth_type:	resd 1
.synth_subtype:	resd 1
.perc_mode:	resd 1
.nr_voices:	resd 1
.nr_drums:	resd 1
.instr_bank_size:	resd 1
.capabilities:	resd 1
.dummies:	resd 19			; Reserved space
.name:		resb 30			; SEQ name
.seqbuf:	resb SEQ_buf_size
.seqdev_name:	resb 256		; Devicename ASCIIZ
endstruc

;;;*
;;;* SEQ_open
;;;*
;;;*  In: SEQ_open *seqdev, dword Type, dword Subtype
;;;*      (*seqdev [eax+0], Type [eax+4], Subtype [eax+8])
;;;*  Out: eax=*_snd (on success); -1 (on error)
;;;*
;;;*  Open SEQ device.
;;;*
%macro SEQ_open 1-3
	extern _SEQ_open
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	section .data
%%seqdev:	db "/dev/sequencer",0
	section .text
	__push %2, %1, dword %%seqdev
	mov eax,esp
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _SEQ_open
%if %0 != 1
	add esp,byte 12
%endif
%endm
%macro SEQ_Open 3
	section .data
%%seqdev:	db %1,0
	section .text
	SEQ_open dword %%seqdev, %2, %3
%endm


;;;*
;;;* SEQ_close
;;;*
;;;*  In: SEQ_close *_snd (eax)
;;;*  Out: eax=0 (on success); -1 (on error)
;;;*
;;;*  Close SEQ device.
;;;*
%macro SEQ_close 1
	extern _SEQ_close
%ifnidni %1,eax
	mov eax,%1
%endif
	call _SEQ_close
%endm


;;;*
;;;* SEQ_bufdump
;;;*
;;;*  In: SEQ_bufdump *_snd (eax)
;;;*  Out: eax=0 (on success)
;;;*
;;;*  Write _seqbuf to sequencer.
;;;*
%macro SEQ_bufdump 1
	extern _SEQ_bufdump
%ifnidni %1,eax
	mov eax,%1
%endif
	call _SEQ_bufdump
%endm


;;;*
;;;* SEQ_sendevent
;;;*
;;;*  In: SEQ_sendevent *_snd, event, event_size
;;;*      (*_snd [eax+0], event [eax+4], event_size [eax+8])
;;;*  Out: -
;;;*
;;;*  Sent event _seqbuf.
;;;*
%macro SEQ_sendevent 1-3
	extern _SEQ_sendevent
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
 %error "Macro `SEQ_sendevent' expects 1 or 3 parameters."
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _SEQ_sendevent
%if %0 = 3
	add esp,byte 12
%endif
%endm


;;;*
;;;* SEQ_writepatch
;;;*
;;;*  In: SEQ_writepatch *_snd, *ptr, len
;;;*      (*_snd [eax+0], *ptr [eax+4], len [eax+8])
;;;*  Out: eax=0 (on success)
;;;*
;;;*  Writes a struc patch_info to [seqdev].
;;;*
%macro SEQ_writepatch 3
	extern _SEQ_writepatch
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
 %error "Macro `SEQ_writepatch' expects 1 or 3 parameters."
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _SEQ_writepatch
%if %0 = 3
	add esp,byte 12
%endif
%endm


;;;*
;;;* SEQ_resetsamples
;;;*
;;;*  In: SEQ_resetsamples fd, synth_info.device
;;;*  Out: eax=0 (on success); eax<0 (on error)
;;;*
;;;*  Remove all samples from device (fd, synth_info.device).
;;;*
%macro SEQ_resetsamples 2
	sys_ioctl %1, dword SNDCTL_SEQ_RESETSAMPLES, %2
%endm


;;;*
;;;* SEQ_needbuf
;;;*
;;;*  In: SEQ_needbuf size
;;;*  Out: -
;;;*
;;;*  Build stackframe for SEQ events.
;;;*
%macro SEQ_needbuf 1
	push ebp
	mov ebp,esp
	sub esp,byte %1
%endm


;;;*
;;;* SEQ_adjbuf
;;;*
;;;*  In: SEQ_adjbuf
;;;*  Out: -
;;;*
;;;*  Remove stackframe.
;;;*
%define SEQ_adjbuf leave


;;;
;;; _SEQ_chn_voice
;;;
;;; %1 != eax !!!!!!!!!!!!!!
%macro _SEQ_chn_voice 5
%ifidni %1,eax
 %error "_SEQ_chn_voice: Please do not use 'eax' as %1."
%endif
%ifidni %1,dword [eax]
 %error "_SEQ_chn_voice: Please do not use '[eax]' as %1."
%endif
%define eventsize 8
	SEQ_needbuf eventsize
	__push eax, %5, %4, %3
	mov [ebp-eventsize+0],byte EV_CHN_VOICE
	mov eax,%1		; *_snd
	mov eax,[eax+snd.device]
	mov [ebp-eventsize+1],al
	mov [ebp-eventsize+2],byte %2
	pop eax
	mov [ebp-eventsize+3],al
	pop eax
	mov [ebp-eventsize+4],al
	pop eax
	mov [ebp-eventsize+5],al
	mov [ebp-eventsize+6],word 0
	;; execute SEQ command
	mov eax,ebp
	sub eax,byte eventsize
	SEQ_sendevent %1, eax, dword eventsize
	pop eax
	SEQ_adjbuf
%endm


;;;
;;; _SEQ_chn_common
;;;
;;; %1 != eax !!!!!!!!!!!!!!
%macro _SEQ_chn_common 6
%ifidni %1,eax
 %error "_SEQ_chn_common: Please do not use 'eax' as %1."
%endif
%ifidni %1,dword [eax]
 %error "_SEQ_chn_common: Please do not use '[eax]' as %1."
%endif
%define eventsize 8
	SEQ_needbuf eventsize
	__push eax, %6, %5, %4, %3
	mov [ebp-eventsize+0],byte EV_CHN_COMMON
	mov eax,%1		; *_snd
	mov eax,[eax+snd.device]
	mov [ebp-eventsize+1],al
	mov [ebp-eventsize+2],byte %2
	pop eax
	mov [ebp-eventsize+3],al
	pop eax
	mov [ebp-eventsize+4],al
	pop eax
	mov [ebp-eventsize+5],al
	pop eax
	mov [ebp-eventsize+6],ax
	;; execute SEQ command
	mov eax,ebp
	sub eax,byte eventsize
	SEQ_sendevent %1, eax, dword eventsize
	pop eax
	SEQ_adjbuf
%endm


;;;*
;;;* SEQ_start_note
;;;*
;;;*  In: SEQ_start_note *_snd, chn, note, vol
;;;*  Out: -
;;;*
;;;*  Start chn with note and vol.
;;;*
%macro SEQ_start_note 4
	_SEQ_chn_voice %1, MIDI_NOTEON, %2, %3, %4
%endm


;;;*
;;;* SEQ_stop_note
;;;*
;;;*  In: SEQ_stop_note *_snd, chn, note, vol
;;;*  Out: -
;;;*
;;;*  Stop chn with note and vol.
;;;*
%macro SEQ_stop_note 4
	_SEQ_chn_voice %1, MIDI_NOTEOFF, %2, %3, %4
%endm


;;;*
;;;* SEQ_pgm_change
;;;*
;;;*  In: SEQ_pgm_change *_snd, chn, patch
;;;*  Out: -
;;;*
;;;*  Set channel to patch.
;;;*
%macro SEQ_pgm_change 3
	_SEQ_chn_common %1, MIDI_PGM_CHANGE, %2, %3, dword 0, dword 0
%endm
%macro SEQ_set_patch 3
	SEQ_pgm_change %1, %2, %3
%endm


%endif	; _seq_h_



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: