;;;
;;; string.h
;;;
;;; created: <Fri May  7 21:13:05 1999>
;;; last modified: <Thu Apr 27 23:11:56 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This defines the macros for calling the string functions in libASM.a
;;;

%ifndef _string_h_
%define _string_h_

;;;*
;;;* GetErrorStr
;;;*
;;;*  In: GetErrorStr nr (eax)
;;;*  Out: eax=Errname
;;;*
;;;*  Convert errornumber to name
;;;*
%macro GetErrorStr 1
	extern _GetErrorStr
%ifnidni %1,eax
	mov eax,%1
%endif
	call _GetErrorStr
%endm


;;;*
;;;* Crc32Init
;;;*
;;;*  In: Crc32Init
;;;*  Out: eax=0xffffffff
;;;*
;;;*  Returns 0xffffffff for initiation of crc32 calculation
;;;*
%macro Crc32Init 0
	extern _Crc32Init
	call _Crc32Init
%endm


;;;*
;;;* Crc32Byte
;;;*
;;;*  In: crc32Byte val, x (val [eax+0], x [eax+4])
;;;*  Out: eax=(val [eax+0], x [eax+4])
;;;*
;;;*  Calculates the crc32 val. for the byte x. Prev. val. is initiation
;;;*  string or a previous calculates crc32 val.
;;;*
%macro Crc32Byte 1-2
	extern _Crc32Byte
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _Crc32Byte
%if %0 = 2
	add esp,byte 8
%endif
%endm


;;;*
;;;* Crc32
;;;*
;;;*  In: Crc32 buf, n (buf [eax+0], n [eax+4])
;;;*  Out: eax=crc32
;;;*
;;;*  Calculates a crc32 checksum of buf, length n
;;;*
%macro Crc32 1-2
	extern _Crc32
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _Crc32
%if %0 = 2
	add esp,byte 8
%endif
%endm


;;;*
;;;* Strncpy
;;;*
;;;*  In: Strncpy src, dest, len (src [eax+0], dest [eax+4], len [eax+8])
;;;*  Out: eax=(src [eax+0], dest [eax+4], len [eax+8])
;;;*
;;;*  Copy len bytes from src to dest
;;;*
%macro Strncpy 1-3
	extern _Strncpy
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
 %error "Macro `Strncpy' expects 1 or 3 parameters."
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _Strncpy
%if %0 = 3
	add esp,byte 12
%endif
%endm


;;;*
;;;* Strcpy
;;;*
;;;*  In: Strcpy src, dest(, dummy)  (src [eax+0], dest [eax+4], dummy [eax+8])
;;;*  Out: eax=(src [eax+0], dest [eax+4], dummy [eax+8])
;;;*
;;;*  Copy ASCIIZ from src to dest (including '\0')
;;;*
%macro Strcpy 1-2
	extern _Strcpy
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push dword 0, %2, %1
	mov eax,esp
%endif
	call _Strcpy
%if %0 = 2
	add esp,byte 12
%endif
%endm


;;;*
;;;* Strlen
;;;*
;;;*  In: Strlen src (eax)
;;;*  Out: eax=length
;;;*
;;;*  Return length ASCIIZ string src (NOT including '\0')
;;;*
%macro Strlen 1
	extern _Strlen
%ifnidni %1,eax
	mov eax,%1
%endif
	call _Strlen
%endm


%endif	; _string_h_



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: