;;;
;;; sys.h
;;;
;;; created: <Mon May 10 14:08:17 1999>
;;; last modified: <Thu May  4 21:17:35 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This defines the macros for calling the sys functions in libASM.a
;;;

%ifndef _sys_h_
%define _sys_h_

;;;*
;;;* ExecSP
;;;*
;;;*  In: ExecSP *filename, **argv, **envp (*filename [eax+0], **argv [eax+4],
;;;*       **envp [eax+8])
;;;*  Out: never returns
;;;*
;;;*  Executes filename. If filename has no '/', we will scan the PATH env
;;;*  for valid dir.
;;;*
%macro ExecSP 1-3
	extern _ExecSP
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
 %error "Macro `ExecSP' expects 1 or 3 parameters."
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _ExecSP
	;; 	add esp,byte 12
%endm

;;;*
;;;* ExecSp
;;;*
;;;*  In: ExecSp 'filename', 'arg1', 'arg2', ...
;;;*  Out: never returns
;;;*
;;;*  Executes filename. If filename has no '/', we will scan the PATH env
;;;*  for valid dir.
;;;*

%macro ExecSp 1-*
	section .data
%%data:
	db 0
%rep %0
	db %1,0
%rotate 1
%endrep
%%argv:
%rep %0
	dd 0
%endrep
	dd 0
	section .text
	pushad
	mov ecx,%0
	mov esi,%%argv
	mov edi,%%data
%%_loop:
	push ecx
	mov ecx,0xffffffff
	mov al,0
	cld
	repne scasb
	mov [esi],edi
	add esi,byte 4
	pop ecx
	loop %%_loop
	popad
	ExecSP dword %%data+1, dword %%argv, dword [_envp]
%endm


;;;*
;;;* Socket
;;;*
;;;*  In: Socket family, type, protokoll
;;;*      (family [eax+0], type [eax+4], protokoll [eax+8])
;;;*  Out: eax=Socket descriptor (on success); eax=-1 (on error)
;;;*
;;;*  Create a socket. See socket(2).
;;;*
%macro Socket 3
	extern _Socket
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
 %error "Macro `Socket' expects 1 or 3 parameters."
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _Socket
%if %0 = 3
 	add esp,byte 12
%endif
%endm


;;;*
;;;* Bind
;;;*
;;;*   In: Bind fd, *umyaddr, addrlen (fd [eax+0], *umyaddr [eax+4], addrlen[eax+8])
;;;*   Out: eax=0 (on success); eax=-1 (on error)
;;;*
;;;*   Bind a name to a socket. See bind(2).
;;;*
%macro Bind 3
	extern _Bind
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
 %error "Macro `Bind' expects 1 or 3 parameters."
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _Bind
%if %0 = 3
 	add esp,byte 12
%endif
%endm


;;;*
;;;* Connect
;;;*
;;;*  In: Connect fd, *uservaddr, addrlen
;;;*      (fd [eax+0], *uservaddr[eax+4], addrlen[eax+8])
;;;*  Out: eax=0 (on success); eax=-1 (on error)
;;;*
;;;*  Connect to a socket. See connect(2).
;;;*
%macro Connect 1-3
	extern _Connect
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
 %error "Macro `Connect' expects 1 or 3 parameters."
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _Connect
%if %0 = 3
 	add esp,byte 12
%endif
%endm


;;;*
;;;* Recv
;;;*
;;;*  In: Recv fd, *buf, len, flags
;;;*      (fd [eax+0], *buf [eax+4], len [eax+8], flags [eax+12])
;;;*  Out: eax=0 (on success); eax=-1 (on error)
;;;*
;;;*  Receive data. See recv(2).
;;;*
%macro Recv 1-4
	extern _Recv
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 4
	__push %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `Recv' expects 1 or 4 parameters."
%endif
	call _Recv
%if %0 = 4
 	add esp,byte 16
%endif
%endm


;;;*
;;;* Send
;;;*
;;;*  In: Send fd, *msg, len, flags
;;;*      (fd [eax+0], *msg [eax+4], len [eax+8], flags [eax+12])
;;;*  Out: eax=0 (on success); eax=-1 (on error)
;;;*
;;;*  Send data. See send(2).
;;;*
%macro Send 4
	extern _Send
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 4
	__push %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `Send' expects 1 or 4 parameters."
%endif
	call _Send
%if %0 = 4
 	add esp,byte 16
%endif
%endm


;;;*
;;;* Sleep
;;;*
;;;*  In: Sleep sec, nsec ([eax+0] timespec)
;;;*  Out: eax=0 (on success); eax=-1 (on error)
;;;*
%macro Sleep 1-2
	extern _Sleep
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1		; This creates timespec (see structs.h)
	mov eax,esp
%endif
	call _Sleep
%if %0 = 2
 	add esp,byte 8
%endif
%endm


%endif	; _sys_h_



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: