;;;
;;; system.h
;;;
;;; created: <02.08.1998>
;;; last modified: <Thu Feb  1 19:04:10 2001>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; You can find the latest version of this file here:
;;;  http://www.fowelin.de/christian/computer/linux
;;;
;;; This is handmade for NASM and for stable kernels...
;;;  (>=2.0.35, 2.2.x, 2.4.x. Don't know <=2.0.35, 2.1.x and 2.3.x)
;;; Informations for 2.4.x
;;;

%ifndef _system_h_
%define _system_h_

;;; Put in the systems kernel version
;;; This is to control the possible change of function parameters more easliy.
;;; Only add conditional assembly code for a changed function(-number) in a new
;;; kernel. Differences in entry.S for old functions should only matter infos.
;%define ___KERNEL_VER___ ___CREATE_VER___(2,4,0)
%define ___KERNEL_VER___ ___CREATE_VER___(___KERNEL_VERSION___,___KERNEL_PATCHLEVEL___,___KERNEL_SUBLEVEL___)
%define ___CREATE_VER___(a,b,c) ((a*1000+b)*1000)+c

;;; Get the defs
;%include "system.def.h"

;;;
;;; Some definitions which are usefull (I think so)
;;;
;%idefine offset
;%idefine ptr
%idefine end
;%define __TEXT__ section .text	; progbits alloc exec nowrite align=16
;%define __DATA__ section .data	; progbits alloc noexec write align=4
;%define __BSS__ section .bss	; nobits alloc noexec write align=4
%idefine True 1			; BOOLEAN
%idefine False 0		; BOOLEAN
%define SYSTEM_CALL_GATE 0x80


;;;
;;; Some useful macros (missing in NASM 0.97)
;;;
%idefine fwait db 0x9b

;;;
;;; multible pushing and popping
;;;
%macro __push 1-*
 %rep %0
	push %1
 %rotate 1
 %endrep
%endmacro

%macro __pop 1-*
 %rep %0
	pop %1
 %rotate 1
 %endrep
%endmacro

;;;
;;; This is for totally free register usage...
;;;  eax will be changed.
;;;  sys_exit does not use the __syscallX macros
;;;
%macro __syscall0 1
	mov eax,%1
	int SYSTEM_CALL_GATE
%endm

%macro __syscall1 2
;	__push ebx, %2
;	mov ebx,[dword esp+0]
;	add esp,byte 4
 %ifnidni %2,ebx
	push ebx
	mov ebx,%2
 %endif
	mov eax,%1
	int SYSTEM_CALL_GATE
 %ifnidni %2,ebx
	pop ebx
 %endif
%endm

%macro __syscall2 3
%ifnidni %2,ebx
	push ebx
%endif
%ifnidni %3,ecx
	push ecx
%endif
%ifnidni %2,ebx
; %ifnidni %3,ebx
;	mov ebx,%2
; %else
	push %2
; %endif
%endif
%ifnidni %3,ecx
	mov ecx,%3
%endif
%ifnidni %2,ebx
; %ifidni %3,ebx
	mov ebx,[dword esp+0]
	add esp,byte 4
; %endif
%endif
	mov eax,%1
	int SYSTEM_CALL_GATE
%ifnidni %3,ecx
	pop ecx
%endif
%ifnidni %2,ebx
	pop ebx
%endif
%endm

%macro __syscall3 4
%assign ___tmp1 0
%assign ___tmp2 0
%ifnidni %2,ebx
	push ebx
%endif
%ifnidni %3,ecx
	push ecx
%endif
%ifnidni %4,edx
	push edx
%endif
%ifnidni %2,ebx
	push %2
 %assign ___tmp1 4
%endif
%ifnidni %3,ecx
	push %3
 %assign ___tmp2 4
%endif

%ifnidni %4,edx
	mov edx,%4
%endif

%ifnidni %2,ebx
	mov ebx,[dword esp+___tmp2]
%endif
%ifnidni %3,ecx
	mov ecx,[dword esp+0]
%endif
%assign ___tmp ___tmp1+___tmp2
%if ___tmp > 0
	add esp,byte ___tmp
%endif
	mov eax,%1
	int SYSTEM_CALL_GATE
%ifnidni %4,edx
	pop edx
%endif
%ifnidni %3,ecx
	pop ecx
%endif
%ifnidni %2,ebx
	pop ebx
%endif
%endm

%macro __syscall4 5
%assign ___tmp1 0
%assign ___tmp2 0
%assign ___tmp3 0
%ifnidni %2,ebx
	push ebx
%endif
%ifnidni %3,ecx
	push ecx
%endif
%ifnidni %4,edx
	push edx
%endif
%ifnidni %5,esi
	push esi
%endif
%ifnidni %2,ebx
	push %2
 %assign ___tmp1 4
%endif
%ifnidni %3,ecx
	push %3
 %assign ___tmp2 4
%endif
%ifnidni %4,edx
	push %4
 %assign ___tmp3 4
%endif

%ifnidni %5,esi
	mov esi,%5
%endif

%ifnidni %2,ebx
	mov ebx,[dword esp+___tmp2+___tmp3]
%endif
%ifnidni %3,ecx
	mov ecx,[dword esp+___tmp3]
%endif
%ifnidni %4,edx
	mov edx,[dword esp+0]
%endif
%assign ___tmp ___tmp1+___tmp2+___tmp3
%if ___tmp > 0
	add esp,byte ___tmp
%endif
	mov eax,%1
	int SYSTEM_CALL_GATE
%ifnidni %5,esi
	pop esi
%endif
%ifnidni %4,edx
	pop edx
%endif
%ifnidni %3,ecx
	pop ecx
%endif
%ifnidni %2,ebx
	pop ebx
%endif
%endm

%macro __syscall5 6
%assign ___tmp1 0
%assign ___tmp2 0
%assign ___tmp3 0
%assign ___tmp4 0
%ifnidni %2,ebx
	push ebx
%endif
%ifnidni %3,ecx
	push ecx
%endif
%ifnidni %4,edx
	push edx
%endif
%ifnidni %5,esi
	push esi
%endif
%ifnidni %6,edi
	push edi
%endif
%ifnidni %2,ebx
	push %2
 %assign ___tmp1 4
%endif
%ifnidni %3,ecx
	push %3
 %assign ___tmp2 4
%endif
%ifnidni %4,edx
	push %4
 %assign ___tmp3 4
%endif
%ifnidni %5,esi
	push %5
 %assign ___tmp4 4
%endif

%ifnidni %6,edi
	mov edi,%6
%endif

%ifnidni %2,ebx
	mov ebx,[dword esp+___tmp2+___tmp3+___tmp4]
%endif
%ifnidni %3,ecx
	mov ecx,[dword esp+___tmp3+___tmp4]
%endif
%ifnidni %4,edx
	mov edx,[dword esp+___tmp4]
%endif
%ifnidni %5,esi
	mov esi,[dword esp+0]
%endif
%assign ___tmp ___tmp1+___tmp2+___tmp3+___tmp4
%if ___tmp > 0
	add esp,byte ___tmp
%endif
	mov eax,%1
	int SYSTEM_CALL_GATE
%ifnidni %6,edi
	pop edi
%endif
%ifnidni %5,esi
	pop esi
%endif
%ifnidni %4,edx
	pop edx
%endif
%ifnidni %3,ecx
	pop ecx
%endif
%ifnidni %2,ebx
	pop ebx
%endif
%endm

%macro __syscall6 7
%assign ___tmp1 0
%assign ___tmp2 0
%assign ___tmp3 0
%assign ___tmp4 0
%assign ___tmp5 0
%ifnidni %2,ebx
	push ebx
%endif
%ifnidni %3,ecx
	push ecx
%endif
%ifnidni %4,edx
	push edx
%endif
%ifnidni %5,esi
	push esi
%endif
%ifnidni %6,edi
	push edi
%endif
%ifnidni %7,ebp
	push ebp
%endif
%ifnidni %2,ebx
	push %2
 %assign ___tmp1 4
%endif
%ifnidni %3,ecx
	push %3
 %assign ___tmp2 4
%endif
%ifnidni %4,edx
	push %4
 %assign ___tmp3 4
%endif
%ifnidni %5,esi
	push %5
 %assign ___tmp4 4
%endif
%ifnidni %6,edi
	push %6
 %assign ___tmp5 4
%endif

%ifnidni %7,edi
	mov edi,%7
%endif

%ifnidni %2,ebx
	mov ebx,[dword esp+___tmp2+___tmp3+___tmp4+___tmp5]
%endif
%ifnidni %3,ecx
	mov ecx,[dword esp+___tmp3+___tmp4+___tmp5]
%endif
%ifnidni %4,edx
	mov edx,[dword esp+___tmp4+___tmp5]
%endif
%ifnidni %5,esi
	mov esi,[dword esp+___tmp5]
%endif
%ifnidni %6,edi
	mov edi,[dword esp+0]
%endif
%assign ___tmp ___tmp1+___tmp2+___tmp3+___tmp4+___tmp5
%if ___tmp > 0
	add esp,byte ___tmp
%endif
	mov eax,%1
	int SYSTEM_CALL_GATE
%ifnidni %7,ebp
	pop ebp
%endif
%ifnidni %6,edi
	pop edi
%endif
%ifnidni %5,esi
	pop esi
%endif
%ifnidni %4,edx
	pop edx
%endif
%ifnidni %3,ecx
	pop ecx
%endif
%ifnidni %2,ebx
	pop ebx
%endif
%endm

	
;
; system call macros
;
;
; [unistd.h]
;  calls [entry.S] -> path/file.c
;  man: ${[unistd.h]/__Nr_/}(2)
;
;  In: ${[unistd.h]/__Nr_/sys_} args
;  Out: eax=0 (on success); -1 (on error)
;

;;;
;;; exit
;;;  calls sys_exit -> kernel/exit.c
;;;  man: _exit(2)
;;;
;;;  In: sys_exit error_code
;;;  Out: never returns
;;;
%macro sys_exit 0-1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
 %if %0 = 0
	mov ebx,0
 %else
 %ifnidni %1,ebx
	mov ebx,%1
 %endif
 %endif
	mov eax,__NR_exit
	int SYSTEM_CALL_GATE
%else
 %error "Function `sys_exit' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; fork
;;;  calls sys_fork -> arch/i386/kernel/process.c
;;;  man: fork(2)
;;;
;;;  In: sys_fork
;;;  Out: eax=PID (parent on success); 0 (child on success); -1 (on error)
;;;
%macro sys_fork 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_fork
%else
 %error "Function `sys_fork' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; read
;;;  calls sys_read -> fs/read_write.c
;;;  man: read(2)
;;;
;;;  In: sys_read fd, *buf, count
;;;  Out: eax=bytes read (on success); -1 (on orror)
;;;
%macro sys_read 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_read, %1, %2, %3
%else
 %error "Function `sys_read' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; write
;;;  calls sys_write -> fs/read_write.c
;;;  man: write(2)
;;;
;;;  In: sys_write fd, *buf, count
;;;  Out: eax=bytes written (on success); -1 (on error)
;;;
%macro sys_write 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_write, %1, %2, %3
%else
 %error "Function `sys_write' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; open
;;;  calls sys_open -> fs/open.c
;;;  man: open(2)
;;;
;;;  In: sys_open *filename, flags, mode
;;;  Out: eax=fd (on success); -1 (on error)
;;;
%macro sys_open 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_open, %1, %2, %3
%else
 %error "Function `sys_open' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; close
;;;  calls sys_close -> fs/open.c
;;;  man: close(2)
;;;
;;;  In: sys_close fd
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_close 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_close, %1
%else
 %error "Function `sys_close' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; waitpid
;;;  calls sys_waitpid -> kernel/exit.c
;;;  man: wait(2)
;;;
;;;  In: sys_waitpid pid, *stat_addr, options
;;;  Out: eax=pid child (on success); -1 (on error)
;;;
%macro sys_waitpid 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_waitpid, %1, %2, %3
%else
 %error "Function `sys_waitpid' may not be supported by selected kernel version."
%endif
%endm
; See waitpid(2)
%macro WIFEXITED 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
	and eax,0xff
%endm
%macro WEXITSTATUS 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
	and eax,0xff00
	shr eax,8
%endm
%macro WIFSIGNALED 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
	dec eax
	and eax,0xffff
%endm
%macro WTERMSIG 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
	and eax,0x7f
%endm
%macro WSTOPSIG 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
	WEXITSTATUS eax
%endm


;;;
;;; creat
;;;  calls sys_creat -> fs/open.c
;;;  man: open(2)
;;;
;;;  In: sys_creat *pathname, mode
;;;  Out: eax=fd (on success); -1 (on error)
;;;
%macro sys_creat 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_creat, %1, %2
%else
 %error "Function `sys_creat' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; link
;;;  calls sys_link -> fs/namei.c
;;;  man: link(2)
;;;
;;;  In: sys_link *oldpath, *newpath
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_link 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_link, %1, %2
%else
 %error "Function `sys_link' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; unlink
;;;  calls sys_unlink -> fs/namei.c
;;;  man: unlink(2)
;;;
;;;  In: sys_unlink *pathname
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_unlink 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_unlink, %1
%else
 %error "Function `sys_unlink' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; execve
;;;  calls sys_execve -> arch/i386/kernel/process.c
;;;  man: execve(2)
;;;
;;;  In: sys_execve *filename, **argv, **envp
;;;      filename: db 'name',0
;;;      argv: dd arg1,arg2,...,0
;;;      envp: dd env1,env2,...,0
;;;      arg1: db 'arg',0
;;;      env1: db 'env',0
;;;  Out: No return on success; eax=-1 (on error)
;;;
%macro sys_execve 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_execve, %1, %2, %3
%else
 %error "Function `sys_execve' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; chdir
;;;  calls sys_chdir -> fs/open.c
;;;  man: chdir(2)
;;;
;;;  In: sys_chdir *filename
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_chdir 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_chdir, %1
%else
 %error "Function `sys_chdir' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; time
;;;  calls sys_time -> kernel/time.c
;;;  man: time(2)
;;;
;;;  In: sys_time *tloc
;;;  Out: eax=time in seconds (success)
;;;
%macro sys_time 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_time, %1
%else
 %error "Function `sys_time' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; mknod
;;;  calls sys_mknod -> fs/namei.c
;;;  man: mknod(2)
;;;
;;;  In: sys_mknod *pathname, mode, dev
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_mknod 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_mknod, %1, %2, %3
%else
 %error "Function `sys_mknod' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; chmod
;;;  calls sys_chmod -> fs/open.c
;;;  man: chmod(2)
;;;
;;;  In: sys_chmod *filename, mode
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_chmod 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_chmod, %1, %2
%else
 %error "Function `sys_chmod' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; lchown
;;;  calls sys_lchown16 -> kernel/uid16.c
;;;  man: chown(2)
;;;
;;;  In: sys_lchown *filename, user, group
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_lchown 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall3 __NR_lchown, %1, %2, %3
%else
 %error "Function `sys_lchown' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; break
;;;  calls sys_ni_syscall -> kernel/sys.c
;;;  man: unimplemented(2)
;;;
;;;  In: sys_break
;;;  Out: eax=-1 (error)
;;;
%macro sys_break 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_break
%else
 %error "Function `sys_break' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; oldstat
;;;  calls sys_stat -> fs/stat.c
;;;  man: stat(2), obsolete(2)
;;;
;;;  In: sys_oldstat *filename, *statbuf
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_oldstat 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_oldstat, %1, %2
%else
 %error "Function `sys_oldstat' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; lseek
;;;  calls sys_lseek -> fs/read_write.c
;;;  man: lseek(2)
;;;
;;;  In: sys_lseek fd, offset, origin
;;;  Out: eax=offset from the file start (on success); -1 (on error)
;;;
%macro sys_lseek 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_lseek, %1, %2, %3
%else
 %error "Function `sys_lseek' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getpid
;;;  calls sys_getpid -> kernel/timer.c
;;;  man: getpid(2)
;;;
;;;  In: sys_getpid
;;;  Out: eax=process ID (success)
;;;
%macro sys_getpid 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_getpid
%else
 %error "Function `sys_getpid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; mount
;;;  calls sys_mount -> fs/super.c
;;;  man: mount(2)
;;;
;;;  In: sys_mount *dev_name, *dir_name, *type, new_flags, *data
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_mount 5
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall5 __NR_mount, %1, %2, %3, %4, %5
%else
 %error "Function `sys_mount' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; oldumount
;;;  calls sys_oldumount -> fs/super.c
;;;  man:  mount(2)
;;;
;;;  In: sys_oldumount *name
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_oldumount 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
 %if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall1 __NR_oldumount, %1
 %else
	__syscall1 __NR_umount, %1
 %endif
%else
 %error "Function `sys_umount' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setuid
;;;  calls sys_setuid16 -> kernel/uid16.c
;;;  man: setuid(2)
;;;
;;;  In: sys_setuid uid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setuid 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_setuid, %1
%else
 %error "Function `sys_setuid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getuid
;;;  calls sys_getuid16 -> kernel/uid16.c
;;;  man: getuid(2)
;;;
;;;  In: sys_getuid
;;;  Out: eax=uid (success)
;;;
%macro sys_getuid 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_getuid
%else
 %error "Function `sys_getuid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; stime
;;;  calls sys_stime -> kernel/time.c
;;;  man: stime(2)
;;;
;;;  In: sys_stime *tptr
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_stime 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_stime, %1
%else
 %error "Function `sys_stime' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; ptrace
;;;  calls sys_ptrace -> arch/i386/kernel/ptrace.c
;;;  man: ptrace(2)
;;;
;;;  In: sys_ptrace request, pid, addr, data
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_ptrace 4
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall4 __NR_ptrace, %1, %2, %3, %4
%else
 %error "Function `sys_ptrace' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; alarm
;;;  calls sys_alarm -> kernel/timer.c
;;;  man: alarm(2)
;;;
;;;  In: sys_alarm seconds
;;;  Out: seconds remaining to alarm;;; 0 if no previous alarm
;;;
%macro sys_seconds 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_seconds, %1
%else
 %error "Function `sys_alarm' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; oldfstat
;;;  calls sys_fstat -> fs/stat.c
;;;  man: stat(2), obsolete(2)
;;;
;;;  In: sys_oldfstat fd, *statbuf
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_oldfstat 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_oldfstat, %1, %2
%else
 %error "Function `sys_oldfstat' may not be supported by selected kernel version."
%endif
%endm

;;;
;;; pause
;;;  calls sys_pause -> arch/i386/kernel/sys_i386.c
;;;  man: pause(2)
;;;
;;;  In: sys_pause
;;;  Out: eax=-1 (error)
;;;
%macro sys_pause 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_pause
%else
 %error "Function `sys_pause' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; utime
;;;  calls sys_utime -> fs/open.c
;;;  man: utime(2)
;;;
;;;  In: sys_utime *filename, *times
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_utime 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_utime, %1, %2
%else
 %error "Function `sys_utime' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; stty
;;;  calls sys_ni_syscall -> kernel/sys.c
;;;  man: unimplemented(2)
;;;
;;;  In: sys_stty
;;;  Out: eax=-1 (error)
;;;
%macro sys_stty 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_stty
%else
 %error "Function `sys_stty' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; gtty
;;;  calls sys_ni_syscall -> kernel/sys.c
;;;  man: unimplemented(2)
;;;
;;;  In: sys_gtty
;;;  Out: eax=-1 (error)
;;;
%macro sys_gtty 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_gtty
%else
 %error "Function `sys_gtty' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; access
;;;  calls sys_access -> fs/open.c
;;;  man: access(2)
;;;
;;;  In: sys_access *filename, mode
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_access 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_access, %1, %2
%else
 %error "Function `sys_access' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; nice
;;;  calls sys_nice -> kernel/sched.c
;;;  man: nice(2)
;;;
;;;  In: sys_nice increment
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_nice 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_nice, %1
%else
 %error "Function `sys_nice' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; ftime
;;;  calls sys_ni_syscall -> kernel/sys.c
;;;  man: ftime(3)
;;;
;;;  In: sys_ftime
;;;  Out: -1 (error)
;;;
%macro sys_ftime 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_ftime
%else
 %error "Function `sys_ftime' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sync
;;;  calls sys_sync -> fs/buffer.c
;;;  man: sync(2)
;;;
;;;  In: sys_sync
;;;  Out: eax=0 (success)
;;;
%macro sys_sync 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_sync
%else
 %error "Function `sys_sync' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; kill
;;;  calls sys_kill -> kernel/signal.c
;;;  man: kill(2)
;;;
;;;  In: sys_kill pid, sig
;;;  Out: 0 (on success); -1 (on error)
;;;
%macro sys_kill 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_kill, %1, %2
%else
 %error "Function `sys_kill' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; rename
;;;  calls sys_rename -> fs/namei.c
;;;  man: rename(2)
;;;
;;;  In: sys_rename *oldpath, *newpath
;;;  Out: 0 (on success); -1 (on error)
;;;
%macro sys_rename 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_rename, %1, %2
%else
 %error "Function `sys_rename' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; mkdir
;;;  calls sys_mkdir -> fs/namei.c
;;;  man: mkdir(2)
;;;
;;;  In: sys_mkdir *pathname, mode
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_mkdir 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_mkdir, %1, %2
%else
 %error "Function `sys_mkdir' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; rmdir
;;;  calls sys_rmdir -> fs/namei.c
;;;  man: rmdir(2)
;;;
;;;  In: sys_rmdir *pathname
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_rmdir 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_rmdir, %1
%else
 %error "Function `sys_rmdir' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; dup
;;;  calls sys_dup -> fs/fcntl.c
;;;  man: dup(2)
;;;
;;;  In: sys_dup oldfd
;;;  Out: eax=newfd (on success); -1 (on error)
;;;
%macro sys_dup 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_dup, %1
%else
 %error "Function `sys_dup' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; pipe
;;;  calls sys_pipe -> arch/i386/kernel/sys_i386.c
;;;  man: pipe(2)
;;;
;;;  In: sys_pipe *fildes
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_pipe 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_pipe, %1
%else
 %error "Function `sys_pipe' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; times
;;;  calls sys_times -> kernel/sys.c
;;;  man: times(2)
;;;
;;;  In: sys_times *tbuf
;;;  Out: eax=0 (success)
;;;
%macro sys_times 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_times, %1
%else
 %error "Function `sys_times' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; prof
;;;  calls sys_ni_syscall -> kernel/sys.c
;;;  man: unimplemented(2)
;;;
;;;  In: sys_prof
;;;  Out: eax=-1 (error)
;;;
%macro sys_prof 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_prof
%else
 %error "Function `sys_prof' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; brk
;;;  calls sys_brk -> mm/mmap.c
;;;  man: brk(2)
;;;
;;;  In: sys_brk brk
;;;  Out: new brk (on success); -1 (on error)
;;;
%macro sys_brk 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_brk, %1
%else
 %error "Function `sys_brk' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setgid
;;;  calls sys_setgid16 -> kernel/uid16.c
;;;  man: setgid(2)
;;;
;;;  In: sys_setgid gid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setgid 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_setgid, %1
%else
 %error "Function `sys_setgid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getgid
;;;  calls sys_getgid16 -> kernel/uid16.c
;;;  man: getgid(2)
;;;
;;;  In: sys_getgid
;;;  Out: eax=gid (success)
;;;
%macro sys_getgid 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_getgid
%else
 %error "Function `sys_getgid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; signal
;;;  calls sys_signal -> kernel/signal.c
;;;  man: signal(2)
;;;
;;;  In: sys_signal sig, *handler
;;;  Out: eax=prev. value of the signal handler (on success); -1 (on error)
;;;
%macro sys_signal 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_signal, %1, %2
%else
 %error "Function `sys_signal' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; geteuid
;;;  calls sys_geteuid16 -> kernel/uid16.c
;;;  man: geteuid(2)
;;;
;;;  In: sys_geteuid
;;;  Out: eax=effective UID of the current process (on success)
;;;
%macro sys_geteuid 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_geteuid
%else
 %error "Function `sys_geteuid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getegid
;;;  calls sys_getegid16 -> kernel/uid16.c
;;;  man: getegid(2)
;;;
;;;  In: sys_getegid
;;;  Out: eax=effective GID of the current process (success)
;;;
%macro sys_getegid 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_getegid
%else
 %error "Function `sys_getegid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; acct
;;;  calls sys_acct -> kernel/acct.c
;;;  man: acct(2)
;;;
;;;  In: sys_acct *name
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_acct 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_acct, %1
%else
 %error "Function `sys_acct' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; phys
;;;  *** removed ***
;;;  man: phys(2)
;;;
%macro sys_phys 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35) && ___KERNEL_VER___ <= ___CREATE_VER___(2,0,36) 
	__syscall0 __NR_phys
%else
 %error "Function `sys_phys' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; umount2
;;;  calls sys_umount -> fs/super.c
;;;  man: mount(2)
;;;
;;;  In: sys_umount2 *name
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_umount2 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall1 __NR_umount2, %1
%else
 %error "Function `sys_umount2' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; lock
;;;  calls sys_ni_syscall -> kernel/sys.c
;;;  man: unimplemented(2)
;;;
;;;  In: sys_lock
;;;  Out: -1 (error)
;;;
%macro sys_lock 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_lock
%else
 %error "Function `sys_lock' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; ioctl
;;;  calls sys_ioctl -> fs/ioctl.c
;;;  man: ioctl(2)
;;;
;;;  In: sys_ioctl filp, cmd, *arg
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_ioctl 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_ioctl, %1, %2, %3
%else
 %error "Function `sys_ioctl' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; fcntl
;;;  calls sys_fcntl -> fs/fcntl.c
;;;  man: fcntl(2)
;;;
;;;  In: sys_fcntl fd, cmd, arg
;;;  Out: eax=see fcntl(2) (on success); -1 (on error)
;;;
%macro sys_fcntl 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_fcntl, %1, %2, %3
%else
 %error "Function `sys_fcntl' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; mpx
;;;  calls sys_ni_syscall -> kernel/sys.c
;;;  man: unimplemented(2)
;;;
;;;  In: sys_mpx
;;;  Out: -1 (error)
;;;
%macro sys_mpx 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_mpx
%else
 %error "Function `sys_mpx' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setpgid
;;;  calls sys_setpgid -> kernel/sys.c
;;;  man: setpgid(2)
;;;
;;;  In: sys_setpgid pid, pgid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setpgid 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_setpgid, %1, %2
%else
 %error "Function `sys_setpgid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; ulimit
;;;  calls sys_ni_syscall -> kernel/sys.c
;;;  man: unimplemented(2)
;;;
;;;  In: sys_ulimit
;;;  Out: eax=-1 (error)
;;;
%macro sys_ulimit 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_ulimit
%else
 %error "Function `sys_ulimit' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; oldolduname
;;;  calls sys_olduname -> arch/i386/kernel/sys_i386.c
;;;  man: obsolete(2)
;;;
;;;  In: sys_oldolduname *name
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_oldolduname 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_oldolduname, %1
%else
 %error "Function `sys_oldolduname' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; umask
;;;  calls sys_umask -> kernel/sys.c
;;;  man: umask(2)
;;;
;;;  In: sys_umask mask
;;;  Out: previous value of the mask (success)
;;;
%macro sys_umask 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_umask, %1
%else
 %error "Function `sys_umask' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; chroot
;;;  calls sys_chroot -> fs/open.c
;;;  man: chroot(2)
;;;
;;;  In: sys_chroot *filename
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_chroot 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_chroot, %1
%else
 %error "Function `sys_chroot' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; ustat
;;;  calls sys_ustat -> fs/super.c
;;;  man: ustat(2)
;;;
;;;  In: sys_ustat dev, *ubuf
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_ustat 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_ustat, %1, %2
%else
 %error "Function `sys_ustat' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; dup2
;;;  calls sys_dup2 -> fs/fcntl.c
;;;  man: dup(2)
;;;
;;;  In: sys_dup2 oldfd, newfd
;;;  Out: eax=new descriptor(on success); -1 (on error)
;;;
%macro sys_dup2 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_dup2, %1, %2
%else
 %error "Function `sys_dup2' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getppid
;;;  calls sys_getppid -> kernel/timer.c
;;;  man: getpid(2)
;;;
;;;  In: sys_getppid
;;;  Out: eax=parent PID (success)
;;;
%macro sys_getppid 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_getppid
%else
 %error "Function `sys_getppid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getpgrp
;;;  calls sys_getpgrp -> kernel/sys.c
;;;  man: setpgid(2)
;;;
;;;  In: sys_getpgrp
;;;  Out: eax=current process group (success)
;;;
%macro sys_getpgrp 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_getpgrp
%else
 %error "Function `sys_getpgrp' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setsid
;;;  calls sys_setsid -> kernel/sys.c
;;;  man: setsid(2)
;;;
;;;  In: sys_setsid
;;;  Out: eax=SID (on success); -1 (on error)
;;;
%macro sys_setsid 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_setsid
%else
 %error "Function `sys_setsid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sigaction
;;;  calls sys_sigaction -> arch/i386/kernel/signal.c
;;;  man: sigaction(2)
;;;
;;;  In: sys_sigaction signum, *action, oldaction
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_sigaction 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_sigaction, %1, %2, %3
%else
 %error "Function `sys_sigaction' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sgetmask
;;;  calls sys_sgetmask -> kernel/signal.c
;;;  man: signal(2)
;;;
;;;  In: sys_sgetmask
;;;  Out: eax=signal mask (success)
;;;
%macro sys_sgetmask 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_sgetmask
%else
 %error "Function `sys_sgetmask' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; ssetmask
;;;  calls sys_ssetmask -> kernel/signal.c
;;;  man: signal(2)
;;;
;;;  In: sys_ssetmask newmask
;;;  Out: eax=old signal mask (success)
;;;
%macro sys_ssetmask 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_ssetmask, %1
%else
 %error "Function `sys_ssetmask' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setreuid
;;;  calls sys_setreuid16 -> kernel/uid16.c
;;;  man: setreuid(2)
;;;
;;;  In: sys_setreuid ruid, euid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setreuid 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_setreuid, %1, %2
%else
 %error "Function `sys_setreuid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setregid
;;;  calls sys_setregid16 -> kernel/uid16.c
;;;  man: setregid(2)
;;;
;;;  In: sys_setregid rgid, egid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setregid 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_setregid, %1, %2
%else
 %error "Function `sys_setregid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sigsuspend
;;;  calls sys_sigsuspend -> arch/i386/kernel/signal.c
;;;  man: sigaction(2)
;;;
;;;  In: sys_sigsuspend restart, oldmask, set
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_sigsuspend 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_sigsuspend, %1, %2, %3
%else
 %error "Function `sys_sigsuspend' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sigpending
;;;  calls sys_sigpending -> kernel/signal.c
;;;  man: sigaction(2)
;;;
;;;  In: sys_sigpendig *set
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_sigpending 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_sigpending, %1
%else
 %error "Function `sys_sigpending' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sethostname
;;;  calls sys_sethostname -> kernel/sys.c
;;;  man: gethostname(2)
;;;
;;;  In: sys_sethostname *name, len
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_sethostname 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_sethostname, %1, %2
%else
 %error "Function `sys_sethostname' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setrlimit
;;;  calls sys_setrlimit -> kernel/sys.c
;;;  man: getrlimit(2)
;;;
;;;  In: sys_setrlimit resource, *rlim
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setrlimit 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_setrlimit, %1, %2
%else
 %error "Function `sys_setrlimit' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getrlimit
;;;  calls sys_old_getrlimit -> kernel/sys.c
;;;  man: getrlimit(2)
;;;
;;;  In: sys_getrlimit resource, *rlim
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_getrlimit 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_getrlimit, %1, %2
%else
 %error "Function `sys_getrlimit' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getrusage
;;;  calls sys_getrusage -> kernel/sys.c
;;;  man: getrlimit(2)
;;;
;;;  In: sys_getrusage who, *usage
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_getrusage 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_getrusage, %1, %2
%else
 %error "Function `sys_getrusage' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; gettimeofday
;;;  calls sys_gettimeofday -> kernel/time.c
;;;  man: gettimeofday(2)
;;;
;;;  In: sys_gettimeofday *tv, *tz
;;;  Out: eax=0, tz and tv will be filled (on success); -1 (on error)
;;;
%macro sys_gettimeofday 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_gettimeofday, %1, %2
%else
 %error "Function `sys_gettimeofday' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; settimeofday
;;;  calls sys_settimeofday -> kernel/time.c
;;;  man: gettimeofday(2)
;;;
;;;  In: sys_settimeofday *tv, *tz
;;;  Out: eax=0(on success); -1 (on error)
;;;
%macro sys_settimeofday 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_settimeofday, %1, %2
%else
 %error "Function `sys_settimeofday' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getgroups
;;;  calls sys_getgroups16 -> kernel/uid16.c
;;;  man: getgroups(2)
;;;
;;;  In: sys_getgroups gidsetsize, *grouplist
;;;  Out: eax=Up to size supplemental groups (on success); -1 (on error)
;;;
%macro sys_getgroups 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_getgroups, %1, %2
%else
 %error "Function `sys_getgroups' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setgroups
;;;  calls sys_setgroups16 -> kernel/uid16.c
;;;  man: getgroups(2)
;;;
;;;  In: sys_setgroups gidsetsize, *grouplist
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setgroups 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_setgroups, %1, %2
%else
 %error "Function `sys_setgroups' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; select
;;;  calls old_select -> arch/i386/kernel/sys_i386.c
;;;  use _newselect
;;;  man: select(2)
;;;
;;;  In: sys_select *buffer
;;;  Out: eax=number of fds (on success); -1 (on error)
;;;
%macro sys_select 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_select, %1
%else
 %error "Function `sys_select' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; symlink
;;;  calls sys_symlink -> fs/namei.c
;;;  man: symlink(2)
;;;
;;;  In: sys_symlink *oldname, *newname
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_symlink 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_symlink, %1, %2
%else
 %error "Function `sys_symlink' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; oldlstat
;;;  calls sys_lstat -> fs/stat.c
;;;  use lstat
;;;  man: stat(2), obsolete(2)
;;;
;;;  In: sys_oldlstat *filename, *statbuf
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_oldlstat 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_oldlstat, %1, %2
%else
 %error "Function `sys_oldlstat' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; readlink
;;;  calls sys_readlink -> fs/stat.c
;;;  man: readlink(2)
;;;
;;;  In: sys_readlink *path, *buf, bufsiz
;;;  Out: eax=number of characters in buf (on success); -1 (on error)
;;;
%macro sys_readlink 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_readlink, %1, %2, %3
%else
 %error "Function `sys_readlink' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; uselib
;;;  calls sys_uselib -> fs/exec.c
;;;  man: uselib(2)
;;;
;;;  In: sys_uselib *library
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_uselib 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_uselib, %1
%else
 %error "Function `sys_uselib' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; swapon
;;;  calls sys_swapon -> mm/swapfile.c
;;;  man: swapon(2)
;;;
;;;  In: sys_swapon *specialfile, swap_flags
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_swapon 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_swapon, %1, %2
%else
 %error "Function `sys_swapon' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; reboot
;;;  calls sys_reboot -> kernel/sys.c
;;;  man: reboot(2)
;;;
;;;  In: sys_reboot magic, magic_too, flag
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_reboot 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_reboot, %1, %2, %3
%else
 %error "Function `sys_reboot' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; readdir
;;;  calls old_readdir -> fs/readdir.c
;;;  man: readdir(2)
;;;
;;;  In: sys_readdir fd, *dirent, count
;;;  Out: eax=0(end of directory, success); 1 (on success); -1 (on error)
;;;
%macro sys_readdir 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_readdir, %1, %2, %3
%else
 %error "Function `sys_readdir' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; mmap
;;;  calls old_mmap -> arch/i386/kernel/sys_i386.c
;;;  man: mmap(2)
;;;
;;;  In: sys_mmap *buffer (dword offset start, dword length, dword prot,
;;; 	                   dword flags, dword fd, dword offset)
;;;  Out: eax=pointer to the mapped area (on success); -1 (on error)
;;;
%macro sys_mmap 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_mmap, %1
%else
 %error "Function `sys_mmap' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; munmap
;;;  calls sys_munmap -> mm/mmap.c
;;;  man: mmap(2)
;;;
;;;  In: sys_munmap addr, len
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_munmap 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_munmap, %1, %2
%else
 %error "Function `sys_munmap' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; truncate
;;;  calls sys_truncate -> fs/open.c
;;;  man: truncate(2)
;;;
;;;  In: sys_truncate *path, length
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_truncate 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_truncate, %1, %2
%else
 %error "Function `sys_truncate' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; ftruncate
;;;  calls sys_ftruncate -> fs/open.c
;;;  man: truncate(2)
;;;
;;;  In: sys_ftruncate fd, length
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_ftruncate 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_ftruncate, %1, %2
%else
 %error "Function `sys_ftruncate' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; fchmod
;;;  calls sys_fchmod -> fs/open.c
;;;  man: chmod(2)
;;;
;;;  In: sys_fchmod fd, mode
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_fchmod 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_fchmod, %1, %2
%else
 %error "Function `sys_fchmod' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; fchown
;;;  calls sys_fchown16 -> kernel/uid16.c
;;;  man: chown(2)
;;;
;;;  In: sys_fchown fd, user, group
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_fchown 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_fchown, %1, %2, %3
%else
 %error "Function `sys_fchown' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getpriority
;;;  calls sys_getpriority -> kernel/sys.c
;;;  man: getpriority(2)
;;;
;;;  In: sys_getpriority which, who
;;;  Out: eax=scheduling priority indicated in which
;;;
%macro sys_getpriority 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_getpriority, %1, %2
%else
 %error "Function `sys_getpriority' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setpriority
;;;  calls sys_setpriority -> kernel/sys.c
;;;  man: setpriority(2)
;;;
;;;  In: sys_setpriority which, who, niceval
;;;  Out: eax=scheduling priority indicated in which
;;;
%macro sys_setpriority 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_setpriority, %1, %2, %3
%else
 %error "Function `sys_setgriority' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; profil
;;;  calls sys_ni_syscall -> kernel/sys.c
;;;  man: profil(2)
;;;
;;;  In: sys_profil
;;;  Out: eax=-1 (error)
;;;
%macro sys_profil 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_profil
%else
 %error "Function `sys_profil' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; statfs
;;;  calls sys_statfs -> fs/open.c
;;;  man: statfs(2)
;;;
;;;  In: sys_statfs *path, *buf
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_statfs 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_statfs, %1, %2
%else
 %error "Function `sys_statfs' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; fstatfs
;;;  calls sys_fstatfs -> fs/open.c
;;;  man: statfs(2)
;;;
;;;  In: sys_fstatfs fd, *buf
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_fstatfs 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_fstatfs, %1, %2
%else
 %error "Function `sys_fstatfs' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; ioperm
;;;  calls sys_ioperm -> arch/i386/kernel/ioport.c
;;;  man: ioperm(2)
;;;
;;;  In: sys_ioperm from, num, turn_on
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_ioperm 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_ioperm, %1, %2, %3
%else
 %error "Function `sys_ioperm' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; socketcall
;;;  calls sys_socketcall -> net/socket.c
;;;  man: socketcall(2)
;;;
;;;  In: sys_socketcall call, *args
;;;  Out: ?
;;;
%macro sys_socketcall 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_socketcall, %1, %2
%else
 %error "Function `sys_socketcall' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; syslog
;;;  calls sys_syslog -> kernel/printk.c
;;;  man: syslog(2)
;;;
;;;  In: sys_syslog type, *char, len
;;;  Out: eax=0/bytes read (on success); -1 (on error)
;;;
%macro sys_syslog 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_syslog, %1, %2, %3
%else
 %error "Function `sys_syslog' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setitimer
;;;  calls sys_setitimer -> kernel/itimer.c
;;;  man: getitimer(2)
;;;
;;;  In: sys_setitimer which, *value, *ovalue
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setitimer 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_setitimer, %1, %2, %3
%else
 %error "Function `sys_settimer' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getitimer
;;;  calls sys_getitimer -> kernel/itimer.c
;;;  man: getitimer(2)
;;;
;;;  In: sys_getitimer which, *value
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_getitimer 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_getitimer, %1, %2
%else
 %error "Function `sys_gettimer' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; stat
;;;  calls sys_newstat -> fs/stat.c
;;;  man: stat(2)
;;;
;;;  In: sys_stat *filename, *statbuf
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_stat 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_stat, %1, %2
%else
 %error "Function `sys_stat' may not be supported by selected kernel version."
%endif
%endm
%macro S_ISLNK 1
	and %1,S_IFMT
	cmp %1,S_IFLNK
%endm
%macro S_ISREG 1
	and %1,S_IFMT
	cmp %1,S_IFREG
%endm
%macro S_ISDIR 1
	and %1,S_IFMT
	cmp %1,S_IFDIR
%endm
%macro S_ISCHR 1
	and %1,S_IFMT
	cmp %1,S_IFCHR
%endm
%macro S_ISBLK 1
	and %1,S_IFMT
	cmp %1,S_IFBLK
%endm
%macro S_ISFIFO 1
	and %1,S_IFMT
	cmp %1,S_IFIFO
%endm
%macro S_ISSOCK 1
	and %1,S_IFMT
	cmp %1,S_IFSOCK
%endm


;;;
;;; lstat
;;;  calls sys_newlstat -> fs/stat.c
;;;  man: stat(2)
;;;
;;;  In: sys_lstat *filename, *statbuf
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_lstat 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_lstat, %1, %2
%else
 %error "Function `sys_lstat' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; fstat
;;;  calls sys_newfstat -> fs/stat.c
;;;  man: stat(2)
;;;
;;;  In: sys_fstat fd, *statbuf
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_fstat 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_fstat, %1, %2
%else
 %error "Function `sys_fstat' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; olduname
;;;  calls sys_uname -> arch/i386/kernel/sys_i386.c
;;;  man: obsolete(2)
;;;
;;;  In: sys_olduname *name
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_olduname 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_olduname, %1
%else
 %error "Function `sys_olduname' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; iopl
;;;  calls sys_iopl -> arch/i386/kernel/ioport.c
;;;  man: iopl(2)
;;;
;;;  In: sys_iopl level
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_iopl 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_iopl, %1
%else
 %error "Function `sys_iopl' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; vhangup
;;;  calls sys_vhangup -> fs/open.c
;;;  man: vhangup(2)
;;;
;;;  In: sys_vhangup
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_vhangup 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_vhangup
%else
 %error "Function `sys_vhangup' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; idle
;;;  calls sys_ni_syscall -> kernel/sys.c
;;;  man: idle(2)
;;;
;;;  In: sys_idle
;;;  Out: eax=-1 (on error)
;;;
%macro sys_idle 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35) && ___KERNEL_VER___ < ___CREATE_VER___(2,3,13)
	__syscall0 __NR_idle
%else
 %error "Function `sys_idle' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; vm86old
;;;  calls sys_vm86old -> arch/i386/kernel/vm86.c
;;;  man: vm86(2)
;;;
;;;  In: sys_vm86old *vm86
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_vm86old 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
 %if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall1 __NR_vm86old, %1
 %else
	__syscall1 __NR_vm86, %1
 %endif
%else
 %error "Function `sys_vm86old' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; wait4
;;;  calls sys_wait4 -> kernel/exit.c
;;;  man: wait(2)
;;;
;;;  In: sys_wait4 pid, *stat_addr, options, *ru
;;;  Out: eax=PID of child which exited (on success); -1 (on error)
;;;
%macro sys_wait4 4
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall4 __NR_wait4, %1, %2, %3, %4
%else
 %error "Function `sys_wait4' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; swapoff
;;;  calls sys_swapoff -> mm/swapfile.c
;;;  man: swapon(2)
;;;
;;;  In: sys_swapoff *specialfile
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_swapoff 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_swapoff, %1
%else
 %error "Function `sys_swapoff' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sysinfo
;;;  calls sys_sysinfo -> kernel/info.c
;;;  man: sysinfo(2)
;;;
;;;  In: sys_sysinfo *info
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_sysinfo 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_sysinfo, %1
%else
 %error "Function `sys_sysinfo' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; ipc
;;;  calls sys_ipc -> arch/i386/kernel/sys_i386.c
;;;  man: ipc(2)
;;;
;;;  In: sys_ipc call, first, second, third, *ptr, fifth
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_ipc 6
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall6 __NR_ipc, %1, %2, %3, %4, %5, %6
%else
 %error "Function `sys_ipc' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; fsync
;;;  calls sys_fsync -> fs/buffer.c
;;;  man: fsync(2)
;;;
;;;  In: sys_fsync *fd
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_fsync 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_fsync, %1
%else
 %error "Function `sys_fsync' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sigreturn
;;;  calls sys_sigreturn -> arch/i386/kernel/signal.c
;;;  _Don't_ call this one
;;;  man: sigreturn(2)
;;;
;;;  In: sys_sigreturn __unused
;;;  Out: never returns
;;;
%macro sys_sigreturn 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_sigreturn, %1
%else
 %error "Function `sys_sigreturn' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; clone
;;;  calls sys_clone -> arch/i386/kernel/process.c
;;;  man: clone(2)
;;;
;;;  In: sys_clone clone_flags, newsp
;;;  Out: eax=PID (parent); 0 (child); -1 (on error)
;;;
%macro sys_clone 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_clone, %1, %2
%else
 %error "Function `sys_clone' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setdomainname
;;;  calls sys_setdomainname -> kernel/sys.c
;;;  man: setdomainname(2)
;;;
;;;  In: sys_setdomainname *name, len
;;;  Out: eax=0 (in success); -1 (on error)
;;;
%macro sys_setdomainname 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_setdomainname, %1, %2
%else
 %error "Function `sys_setdomainename' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; uname
;;;  calls sys_newuname -> kernel/sys.c
;;;  man: uname(2)
;;;
;;;  In: sys_uname *name
;;;  Out: eax=0 (in success); -1 (on error)
;;;
%macro sys_uname 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_uname, %1
%else
 %error "Function `sys_uname' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; modify_ldt
;;;  calls sys_modify_ldt -> arch/i386/kernel/ldt.c
;;;  man: modify_ldt(2)
;;;
;;;  In: sys_modify_ldt func, *ptr, bytecount
;;;  Out: eax=bytes read (reading); 0 (writing); -1 (on error)
;;;
%macro sys_modify_ldt 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_modify_ldt, %1, %2, %3
%else
 %error "Function `sys_modify_ldt' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; adjtimex
;;;  calls sys_adjtimex -> kernel/time.c
;;;  man: adjtimex(2)
;;;
;;;  In: sys_adjtimex *txc_p
;;;  Out: eax=buf.status (on success); -1 (on error)
;;;
%macro sys_adjtimex 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_adjtimex, %1
%else
 %error "Function `sys_adjtimex' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; mprotect
;;;  calls sys_mprotect -> mm/mprotect.c
;;;  man: mprotect(2)
;;;
;;;  In: sys_mprotect start, len, prot
;;;  Out: eax=0 (success); -1 (error)
;;;
;;; old:
;;;	__push ebx, ecx, edx, %1, %2, %3
;;;	mov eax,__NR_mprotect
;;;	mov ebx,[dword esp+8]
;;;	and ebx,PAGE_MASK
;;;	mov ecx,[dword esp+4]
;;;	add ecx,PAGE_SIZE-1
;;;	and ecx,PAGE_MASK
;;;	mov edx,[dword esp+0]
;;;	add esp,byte 12
;;;	int SYSTEM_CALL_GATE
;;;	__pop edx, ecx, ebx
;;;
%macro sys_mprotect 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_mprotect, %1, %2, %3
%else
 %error "Function `sys_mprotect' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sigprocmask
;;;  calls sys_sigprocmask -> kernel/signal.c
;;;  man: sigprocmask(2)
;;;
;;;  In: sys_sigprocmask how, *set, *oset
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_sigprocmask 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_sigprocmask, %1, %2, %3
%else
 %error "Function `sys_sigprocmask' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; create_module
;;;  calls sys_create_module -> kernel/module.c
;;;  man: create_module(2)
;;;
;;;  In: sys_create_module *module_name, size
;;;  Out: eax=adress of the module (on success); -1 (on error)
;;;
%macro sys_create_module 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_create_module, %1, %2
%else
 %error "Function `sys_create_module' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; init_module
;;;  calls sys_init_module -> kernel/module.c
;;;  man: init_module(2)
;;;
;;;  In: sys_init_module *module_name, *code, codesize, *routines, *symtab
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_init_module 5
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall5 __NR_init_module, %1, %2, %3, %4, %5
%else
 %error "Function `sys_init_module' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; delete_module
;;;  calls sys_delete_module -> kernel/module.c
;;;  man: delete_module(2)
;;;
;;;  In: sys_delete_module *module_name
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_delete_module 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_delete_module, %1
%else
 %error "Function `sys_delete_module' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; get_kernel_syms
;;;  calls sys_get_kernel_syms -> kernel/module.c
;;;  man: get_kernel_syms(2)
;;;
;;;  In: sys_get_kernel_syms *table
;;;  Out: eax=number of symbols returned (success)
;;;
%macro sys_get_kernel_syms 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_get_kernel_syms, %1
%else
 %error "Function `sys_get_kernel_syms' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; quotactl
;;;  calls sys_quotactl -> fs/dquot.c
;;;  man: -
;;;
;;;  In: sys_quotactl cmd, *special, id, addr
;;;  Out: eax=ret (success); -1 (error)
;;;
%macro sys_quotactl 4
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall4 __NR_quotactl, %1, %2 ,%3 ,%4
%else
 %error "Function `sys_quotactl' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getpgid
;;;  calls sys_getpgid -> kernel/sys.c
;;;  man: -
;;;
;;;  In: sys_getpgid pid
;;;  Out: eax=PGID (on success); -1 (on error)
;;;
%macro sys_getpgid 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_getpgid, %1
%else
 %error "Function `sys_getpgid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; fchdir
;;;  calls sys_fchdir -> fs/open.c
;;;  man: chdir(2)
;;;
;;;  In: sys_fchdir fd
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_fchdir 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_fchdir, %1
%else
 %error "Function `sys_fchdir' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; bdflush
;;;  calls sys_bdflush -> fs/buffer.c
;;;  man: bdflush(2)
;;;
;;;  In: sys_bdflush func, data
;;;  Out: eax=never returns/0 (on success); -1 (on error)
;;;
%macro sys_bdflush 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_bdflush, %1, %2
%else
 %error "Function `sys_bdflush' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sysfs
;;;  calls sys_sysfs -> fs/super.c
;;;  man: sysfs(2)
;;;
;;;  In: sys_sysfs option, ...
;;;  Out: eax=fs index/number of current fs/0 (on success); -1 (on error)
;;;
%macro sys_sysfs 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_sysfs, %1, %2, %3
%else
 %error "Function `sys_sysfs' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; personality
;;;  calls sys_personality -> kernel/exec_domain.c
;;;  man: personality(2)
;;;
;;;  In: sys_personality personality
;;;  Out: eax=-1 (on error)
;;;
%macro sys_personality 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_personality, %1
%else
 %error "Function `sys_personality' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; afs_syscall
;;;  calls sys_ni_syscall -> kernel/sys.c
;;;  man: unimplemented(2)
;;;
;;;  In: sys_afs_syscall
;;;  Out: eax=?
;;;
%macro sys_afs_syscall 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_afs_syscall
%else
 %error "Function `sys_afs_syscall' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setfsuid
;;;  calls sys_setfsuid16 -> kernel/uid16.c
;;;  man: setfsuid(2)
;;;
;;;  In: sys_setfsuid uid
;;;  Out: eax=prev value of fsuid (on success); current value of fsuid (on error)
;;;
%macro sys_setfsuid 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_setfsuid, %1
%else
 %error "Function `sys_setfsuid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setfsgid
;;;  calls sys_setfsgid16 -> kernel/uid16.c
;;;  man: setfsgid(2)
;;;
;;;  In: sys_setfsgid gid
;;;  Out: eax=prev value of fsgid (on success); current value of fsgid (on error)
;;;
%macro sys_setfsgid 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_setfsgid, %1
%else
 %error "Function `sys_setfsgid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; _llseek
;;;  calls sys_llseek -> fs/read_write.c
;;;  man: llseek(2)
;;;
;;;  In: sys__llseek fd, offset_high, offset_low, *result, origin
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys__llseek 5
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall5 __NR__llseek, %1, %2, %3, %4, %5
%else
 %error "Function `sys__llseek' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getdents
;;;  calls sys_getdents -> fs/readdir.c
;;;  man: getdents(2)
;;;
;;;  In: sys_getdents fd, *dirent, count
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_getdents 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_getdents, %1, %2, %3
%else
 %error "Function `sys_getdents' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; _newselect
;;;  calls sys_select -> fs/select.c
;;;  man: select(2)
;;;
;;;  In: sys__newselect n, *inp, *outp, *exp, *tvp
;;;  Out: eax=number of returned fds (on success); -1 (on error)
;;;
%macro sys__newselect 5
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall5 __NR__newselect, %1, %2, %3, %4, %5
%else
 %error "Function `sys__newselect' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; flock
;;;  calls sys_flock -> fs/locks.c
;;;  man: flock(2)
;;;
;;;  In: sys_flock fd, cmd
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_flock 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_flock, %1, %2
%else
 %error "Function `sys_flock' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; msync
;;;  calls sys_msync -> mm/filemap.c
;;;  man: msync(2)
;;;
;;;  In: sys_msync start, len, flags
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_msync 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_msync, %1, %2, %3
%else
 %error "Function `sys_msync' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; readv
;;;  calls sys_readv -> fs/read_write.c
;;;  man: readv(2)
;;;
;;;  In: sys_readv fd, *vector, count
;;;  Out: eax=bytes read (on success); -1 (on error)
;;;
%macro sys_readv 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_readv, %1, %2, %3
%else
 %error "Function `sys_readv' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; writev
;;;  calls sys_writev -> fs/read_write.c
;;;  man: writev(2)
;;;
;;;  In: sys_writev fd, *vector, count
;;;  Out: eax=bytes written (on success); -1 (on error)
;;;
%macro sys_writev 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_writev, %1, %2, %3
%else
 %error "Function `sys_writev' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getsid
;;;  calls sys_getsid -> kernel/sys.c
;;;  man: getsid(2)
;;;
;;;  In: sys_getsid pid
;;;  Out: eax=PID (on success); -1 (on error)
;;;
%macro sys_getsid 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_getsid, %1
%else
 %error "Function `sys_getsid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; fdatasync
;;;  calls sys_fdatasync -> fs/buffer.c
;;;  man: -
;;;
;;;  In: sys_fdatasync fd
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_fdatasync 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_fdatasync, %1
%else
 %error "Function `sys_fdatasync' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; _sysctl
;;;  calls sys_sysctl -> kernel/sysctl.c
;;;  man: sysctl(2)
;;;
;;;  In: sys__sysctl *args
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys__sysctl 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR__sysctl, %1
%else
 %error "Function `sys__sysctl' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; mlock
;;;  calls sys_mlock -> mm/mlock.c
;;;  man: mlock(2)
;;;
;;;  In: sys_mlock start, len
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_mlock 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_mlock, %1, %2
%else
 %error "Function `sys_mlock' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; munlock
;;;  calls sys_munlock -> mm/mlock.c
;;;  man: munlock(2)
;;;
;;;  In: sys_munlock start, len
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_munlock 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_munlock, %1, %2
%else
 %error "Function `sys_munlock' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; mlockall
;;;  calls sys_mlockall -> mm/mlock.c
;;;  man: mlockall(2)
;;;
;;;  In: sys_mlockall flags
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_mlockall 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_mlockall, %1
%else
 %error "Function `sys_mlockall' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; munlockall
;;;  calls sys_munlockall -> mm/mlock.c
;;;  man: munlockall(2)
;;;
;;;  In: sys_munlockall
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_munlockall 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_munlockall
%else
 %error "Function `sys_munlockall' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sched_setparam
;;;  calls sys_sched_setparam -> kernel/sched.c
;;;  man: sched_setparam(2)
;;; 
;;;  In: sys_sched_setparam pid, *param
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_sched_setparam 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_sched_setparam, %1, %2
%else
 %error "Function `sys_sched_setparam' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sched_getparam
;;;  calls sys_sched_getparam -> kernel/sched.c
;;;  man: sched_setparam(2)
;;;
;;;  In: sys_sched_getparam pid, *param
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_sched_getparam 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_sched_getparam, %1, %2
%else
 %error "Function `sys_sched_getparam' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sched_setscheduler
;;;  calls sys_sched_setscheduler -> kernel/sched.c
;;;  man: sched_setscheduler(2)
;;;
;;;  In: sys_sched_setscheduler pid, policy, *param
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_sched_setscheduler 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_sched_setscheduler, %1, %2, %3
%else
 %error "Function `sys_sched_setscheduler' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sched_getscheduler
;;;  calls sys_sched_getscheduler -> kernel/sched.c
;;;  man: sched_setscheduler(2)
;;;
;;;  In: sys_sched_getscheduler pid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_sched_getscheduler 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_sched_getscheduler, %1
%else
 %error "Function `sys_sched_getscheduler' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sched_yield
;;;  calls sys_sched_yield -> kernel/sched.c
;;;  man: sched_yield(2)
;;;
;;;  In: sys_sched_yield
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_sched_yield 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall0 __NR_sched_yield
%else
 %error "Function `sys_sched_yield' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sched_get_priority_max
;;;  calls sys_sched_get_priority_max -> kernel/sched.c
;;;  man: get_priority_max(2)
;;;
;;;  In: sys_sched_get_priority_max policy
;;;  Out: eax=maximum priority value for policy (on success); -1 (on error)
;;;
%macro sys_sched_get_priority_max 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_sched_get_priority_max, %1
%else
 %error "Function `sys_sched_get_priority_max' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sched_get_priority_min
;;;  calls sys_sched_get_priority_min -> kernel/sched.c
;;;  man: get_priority_max(2)
;;;
;;;  In: sys_sched_get_priority_min policy
;;;  Out: eax=minimum priority value for policy (on success); -1 (on error)
;;;
%macro sys_sched_get_priority_min 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall1 __NR_sched_get_priority_min, %1
%else
 %error "Function `sys_sched_get_priority_min' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sched_rr_get_interval
;;;  calls sys_sched_rr_get_interval -> kernel/sched.c
;;;  man: rr_get_interval(2)
;;;
;;;  In: sys_sched_rr_get_interval pid, *interval
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_sched_rr_get_interval 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_sched_rr_get_interval, %1, %2
%else
 %error "Function `sys_sched_rr_get_interval' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; nanosleep
;;;  calls sys_nanosleep -> kernel/timer.c
;;;  man: nanosleep(2)
;;;
;;;  In: sys_nanosleep *rqtp, *rmtp
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_nanosleep 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall2 __NR_nanosleep, %1, %2
%else
 %error "Function `sys_nanosleep' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; mremap
;;;  calls sys_mremap -> mm/mremap.c
;;;  man: mremap(2)
;;;
;;;  In: sys_mremap addr, old_len, new_len, flags
;;;  Out: eax=pointer to vmemory (on success); -1 (on error)
;;;
%macro sys_mremap 4
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall4 __NR_mremap, %1, %2, %3, %4
%else
 %error "Function `sys_mremap' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setresuid
;;;  calls sys_setresuid16 -> kernel/uid16.c
;;;  man: setresuid(2)
;;;
;;;  In: sys_setresuid ruid, euid, suid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setresuid 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall3 __NR_setresuid, %1, %2, %3
%else
 %error "Function `sys_setresuid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getresuid
;;;  calls sys_getresuid16 -> kernel/uid16.c
;;;  man: getresuid(2)
;;; In: sys_getresuid *ruid, *euid, *suid
;;; Out: eax= retval(?)
%macro sys_getresuid 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall3 __NR_getresuid, %1, %2, %3
%else
 %error "Function `sys_getresuid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; vm86
;;;  calls sys_vm86 -> arch/i386/kernel/vm86.c
;;;  man: vm86(2)
;;;
;;;  In: sys_vm86 subfunction,  *vm86
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_vm86 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall2 __NR_vm86, %1, %2
%else
 %error "Function `sys_vm86' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; query_module
;;;  calls sys_query_module -> kernel/module.c
;;;  man: query_module(2)
;;;
;;;  In: sys_query_module *name_user, which, *buf, bufsize, *ret
;;;  Out: eax=eax=0 (on success); -1 (on error)
;;;
%macro sys_query_module 5
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall5 __NR_query_module, %1, %2, %3, %4, %5
%else
 %error "Function `sys_query_module' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; poll
;;;  calls sys_poll -> fs/select.c
;;;  man: poll(2)
;;;
;;;  In: sys_poll *ufds, nfds, timeout
;;;  Out: eax>=0 (on success); -1 (on error)
;;;
%macro sys_poll 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall3 __NR_poll, %1, %2, %3
%else
 %error "Function `sys_poll' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; nfsservctl
;;;  calls sys_nfsservctl -> fs/filesystems.c
;;;  man: nfsservctl(2)
;;;
;;;  In: sys_nfsservctl cmd, *argp, *resp
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_nfsservctl 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall3 __NR_nfsservctl, %1, %2, %3
%else
 %error "Function `sys_nfsservctl' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setresgid
;;;  calls sys_setresgid16 -> kernel/uid16.c
;;;  man: setresuid(2)
;;;
;;;  In: sys_setresgid rgid, egid, sgid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setresgid 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall3 __NR_setresgid, %1, %2, %3
%else
 %error "Function `sys_setresgid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getresgid
;;;  calls sys_getresgid16 -> kernel/uid16.c
;;;  man: getresuid(2)
;;;
;;;  In: sys_getresgid *rgid, *egid, *sgid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_getresgid 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall3 __NR_getresgid, %1, %2, %3
%else
 %error "Function `sys_getresgid' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; prctl
;;;  calls sys_prctl -> kernel/sys.c
;;;  man: prctl(2)
;;;
;;;  In: sys_prctl option, arg2, arg3, arg4, arg5
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_prctl 5
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall5 __NR_prctl, %1, %2, %3, %4, %5
%else
 %error "Function `sys_prctl' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; rt_sigreturn
;;;  calls sys_rt_sigreturn -> arch/i386/kernel/signal.c
;;;  man: ?
;;;
;;;  In: sys_rt_sigreturn __unused
;;;  Out: eax=(?)
;;;
%macro sys_rt_sigreturn 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall1 __NR_rt_sigreturn, %1
%else
 %error "Function `sys_rt_sigreturn' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; rt_sigaction
;;;  calls sys_rt_sigaction -> kernel/signal.c
;;;  man: ?
;;;
;;;  In: sys_rt_sigaction sig, *act, *oact, sigsetsize
;;;  Out: eax=(?)
;;;
%macro sys_rt_sigaction 4
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall4 __NR_rt_sigaction, %1, %2, %3, %4
%else
 %error "Function `sys_rt_sigaction' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; rt_sigprocmask
;;;  calls sys_rt_sigprocmask -> kernel/signal.c
;;;  man: ?
;;;
;;;  In: sys_rt_sigprocmask how, *set, *oset, sigsetsize
;;;  Out: eax=(?)
;;;
%macro sys_rt_sigprocmask 4
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall4 __NR_rt_sigprocmask, %1, %2, %3, %4
%else
 %error "Function `sys_rt_sigprocmask' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; rt_sigpending
;;;  calls sys_rt_sigpending -> kernel/signal.c
;;;  man: ?
;;;
;;;  In: sys_rt_sigpending *set, sigsetsize
;;;  Out: eax=(?)
;;;
%macro sys_rt_sigpending 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall2 __NR_rt_sigpending, %1, %2
%else
 %error "Function `sys_rt_sigpending' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; rt_sigtimedwait
;;;  calls sys_rt_sigtimedwait -> kernel/signal.c
;;;  man: ?
;;;
;;;  In: sys_rt_sigtimedwait *uthese, *uinfo, *uts, sigsetsize
;;;  Out: eax=(?)
;;;
%macro sys_rt_sigtimedwait 4
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall4 __NR_rt_sigtimedwait, %1, %2, %3, %4
%else
 %error "Function `sys_rt_sigtimedwait' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; rt_sigqueueinfo
;;;  calls sys_rt_sigqueueinfo -> kernel/signal.c
;;;  man: ?
;;;
;;;  In: sys_rt_sigqueueinfo pid, sig, *uinfo
;;;  Out: eax=(?)
;;;
%macro sys_rt_sigqueueinfo 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall3 __NR_rt_sigqueueinfo, %1, %2, %3
%else
 %error "Function `sys_rt_sigqueueinfo' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; rt_sigsuspend
;;;  calls sys_rt_sigsuspend -> arch/i386/kernel/signal.c
;;;  man: ?
;;;
;;;  In: sys_rt_sigsuspend *unewset, sigsetsize
;;;  Out: eax=(?)
;;;
%macro sys_rt_sigsuspend 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall2 __NR_rt_sigsuspend, %1, %2
%else
 %error "Function `sys_rt_sigsuspend' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; pread
;;;  calls sys_pread -> fs/read_write.c
;;;  man: pread(2)
;;;
;;;  In: sys_pread fd, *buf, count, pos
;;;  Out: eax>=0 (on success); -1 (on error)
;;;
%macro sys_pread 4
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall4 __NR_pread, %1, %2, %3, %4
%else
 %error "Function `sys_pread' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; pwrite
;;;  calls sys_pwrite -> fs/read_write.c
;;;  man: pread(2)
;;;
;;;  In: sys_pwrite fd, *buf, count, pos
;;;  Out: eax>=0 (on success); -1 (on error)
;;;
%macro sys_pwrite 4
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall4 __NR_pwrite, %1, %2, %3, %4
%else
 %error "Function `sys_pwrite' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; chown
;;;  calls sys_chown16 -> kernel/uid16.c
;;;  man: chown(2)
;;;
;;;  In: sys_chown *filename, user, group
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_chown 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,0,35)
	__syscall3 __NR_chown, %1, %2, %3
%else
 %error "Function `sys_chown' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getcwd
;;;  calls sys_getcwd -> fs/dcache.c
;;;  man: getcwd(2)
;;;
;;;  In: sys_getcwd *buf, size
;;;  Out: eax=*buf (on success); -1 (on error)
;;;
%macro sys_getcwd 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall2 __NR_getcwd, %1, %2
%else
 %error "Function `sys_getcwd' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; capget
;;;  calls sys_capget -> kernel/capability.c
;;;  man: capget(2)
;;;
;;;  In: sys_capget header, dataptr
;;;  Out: eax=(?)
;;;
%macro sys_capget 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall2 __NR_capget, %1, %2
%else
 %error "Function `sys_capget' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; capset
;;;  calls sys_capset -> kernel/capability.c
;;;  man: capget(2)
;;;
;;;  In: sys_capset header, data
;;;  Out: eax=(?)
;;;
%macro sys_capset 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall2 __NR_capset, %1, %2
%else
 %error "Function `sys_capset' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sigaltstack
;;;  calls sys_sigaltstack -> arch/i386/kernel/signal.c
;;;  man: ?
;;;
;;;  In: sys_sigaltstack *uss, *uoss
;;;  Out: eax=(?)
;;;
%macro sys_sigaltstack 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall2 __NR_sigaltstack, %1, %2
%else
 %error "Function `sys_sigaltstack' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; sendfile
;;;  calls sys_sendfile -> mm/filemap.c
;;;  man: sendfile(2)
;;;
;;;  In: sys_sendfile out_fd, in_fd, *offset, count
;;;  Out: eax>=0 (on success); -1 (on error)
;;;
%macro sys_sendfile 4
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall4 __NR_sendfile, %1, %2, %3, %4
%else
 %error "Function `sys_sendfile' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getpmsg
;;;  calls sys_ni_syscall -> kernel/sys.c
;;;  man: ?
;;;
;;;  In: sys_getpmsg
;;;  Out: eax=-1 (error)
;;;
%macro sys_getpmsg 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall0 __NR_getpmsg
%else
 %error "Function `sys_getpmsg' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; putpmsg
;;;  calls sys_ni_syscall -> kernel/sys.c
;;;  man: ?
;;;
;;;  In: sys_putpmsg
;;;  Out: eax=-1 (error)
;;;
%macro sys_putpmsg 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall0 __NR_putpmsg
%else
 %error "Function `sys_putpmsg' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; vfork
;;;  calls sys_vfork -> arch/i386/kernel/process.c
;;;  man: fork(2)
;;;
;;;  In: sys_vfork
;;;  Out: eax=PID (parent on success); 0 (child on success); -1 (on error)
;;;
%macro sys_vfork 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,2,0)
	__syscall0 __NR_vfork
%else
 %error "Function `sys_vfork' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; ugetrlimit
;;;  calls sys_getrlimit -> kernel/sys.c
;;;  man: getrlimit(2)
;;;
;;;  In: sys_ugetrlimit resource, *rlim
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_ugetrlimit 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall2 __NR_ugetrlimit, %1, %2
%else
 %error "Function `sys_ugetrlimit' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; mmap2
;;;  calls sys_mmap2 -> arch/i386/kernel/sys_i386.c
;;;  man: mmap(2)
;;;
;;;  In: sys_mmap2 addr, len, prot, flags, fd, pgoff
;;;  Out: eax=ptr (on success); -1 (on error)
;;;
%macro sys_mmap2 6
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall6 __NR_mmap2, %1, %2, %3, %4, %5, %6
%else
 %error "Function `sys_mmap2' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; truncate64
;;;  calls sys_truncate64 -> fs/open.c
;;;  man: truncate(2)
;;;
;;;  In: sys_truncate64 *path, length
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_truncate64 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall2 __NR_truncate64, %1, %2
%else
 %error "Function `sys_truncate64' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; ftruncate64
;;;  calls sys_ftruncate64 -> fs/open.c
;;;  man: truncate(2)
;;;
;;;  In: sys_ftruncate64 fd, length
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_ftruncate64 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall2 __NR_ftruncate64, %1, %2
%else
 %error "Function `sys_ftruncate64' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; stat64
;;;  calls sys_stat64 -> fs/stat.c
;;;  man: stat(2)
;;;
;;;  In: sys_stat64 *filename, *statbuf, flags
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_stat64 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall3 __NR_stat64, %1, %2, %3
%else
 %error "Function `sys_stat64' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; lstat64
;;;  calls sys_lstat64 -> fs/stat.c
;;;  man: stat(2)
;;;
;;;  In: sys_lstat64 *filename, *statbuf, flags
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_lstat64 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall3 __NR_lstat64, %1, %2, %3
%else
 %error "Function `sys_lstat64' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; fstat64
;;;  calls sys_fstat64 -> fs/stat.c
;;;  man: stat(2)
;;;
;;;  In: sys_fstat64 fd, *statbuf, flags
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_fstat64 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall3 __NR_fstat64, %1, %2, %3
%else
 %error "Function `sys_fstat64' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; lchown32
;;;  calls sys_lchown -> fs/open.c
;;;  man: lchown(2)
;;;
;;;  In: sys_lchown32 *filename, user, group
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_lchown32 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall3 __NR_lchown32, %1, %2, %3
%else
 %error "Function `sys_lchown32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getuid32
;;;  calls sys_getuid -> kernel/timer.c
;;;  man: getuid(2)
;;;
;;;  In: sys_getuid32
;;;  Out: eax=uid
;;;
%macro sys_getuid32 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall0 __NR_getuid32
%else
 %error "Function `sys_getuid32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getgid32
;;;  calls sys_getgid -> kernel/timer.c
;;;  man: getgid(2)
;;;
;;;  In: sys_getgid32
;;;  Out: eax=gid
;;;
%macro sys_getgid32 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall0 __NR_getgid32
%else
 %error "Function `sys_getgid32' may not be supported by selected kernel version."
%endif
%endm

	
;;;
;;; geteuid32
;;;  calls sys_geteuid -> kernel/timer.c
;;;  man: getuid(2)
;;;
;;;  In: sys_geteuid32
;;;  Out: eax=euid
;;;
%macro sys_geteuid32 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall0 __NR_geteuid32
%else
 %error "Function `sys_geteuid32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getegid32
;;;  calls sys_getegid -> kernel/timer.c
;;;  man: getgid(2)
;;;
;;;  In: sys_getegid32
;;;  Out: eax=egid
;;;
%macro sys_getegid32 0
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall0 __NR_getegid32
%else
 %error "Function `sys_getegid32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setreuid32
;;;  calls sys_setreuid -> kernel/sys.c
;;;  man: setreuid(2)
;;;
;;;  In: sys_setreuid32 ruid, euid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setreuid32 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall2 __NR_setreuid32, %1, %2
%else
 %error "Function `sys_setreuid32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setregid32
;;;  calls sys_setregid -> kernel/sys.c
;;;  man: setregid(2)
;;;
;;;  In: sys_setregid32 rgid, egid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setregid32 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall2 __NR_setregid32, %1, %2
%else
 %error "Function `sys_setregid32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getgroups32
;;;  calls sys_getgroups -> kernel/sys.c
;;;  man: getgroups(2)
;;;
;;;  In: sys_getgroups32 gidsetsize, *grouplist
;;;  Out: eax=gid (on success); -1 (on error)
;;;
%macro sys_getgroups32 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall2 __NR_getgroups32, %1, %2
%else
 %error "Function `sys_getgroups32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setgroups32
;;;  calls sys_setgroups -> kernel/sys.c
;;;  man: getgroups(2)
;;;
;;;  In: sys_setgroups32 gidsetsize, *grouplist
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setgroups32 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall2 __NR_setgroups32, %1, %2
%else
 %error "Function `sys_setgroups32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; fchown32
;;;  calls sys_fchown -> fs/open.c
;;;  man: fchown(2)
;;;
;;;  In: sys_fchown32 fd, user, group
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_fchown32 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall3 __NR_fchown32, %1, %2, %3
%else
 %error "Function `sys_fchown32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setresuid32
;;;  calls sys_setresuid -> kernel/sys.c
;;;  man: setresuid(2)
;;;
;;;  In: sys_setresuid32 ruid, euid, suid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setresuid32 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall3 __NR_setresuid32, %1, %2, %3
%else
 %error "Function `sys_setresuid32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getresuid32
;;;  calls sys_getresuid -> kernel/sys.c
;;;  man: getresuid(2)
;;;
;;;  In: sys_getresuid32 *ruid, *euid, *suid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_getresuid32 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall3 __NR_getresuid32, %1, %2, %3
%else
 %error "Function `sys_getresuid32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setresgid32
;;;  calls sys_setresgid -> kernel/sys.c
;;;  man: setresuid(2)
;;;
;;;  In: sys_setresgid32 rgid, egid, sgid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setresgid32 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall3 __NR_setresgid32, %1, %2, %3
%else
 %error "Function `sys_setresgid32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getresgid32
;;;  calls sys_getresgid -> kernel/sys.c
;;;  man: getresuid(2)
;;;
;;;  In: sys_getresgid32 *rgid, *egid, *sgid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_getresgid32 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall3 __NR_getresgid32, %1, %2, %3
%else
 %error "Function `sys_getresgid32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; chown32
;;;  calls sys_chown -> fs/open.c
;;;  man: chown(2)
;;;
;;;  In: sys_chown32 *filename, user, group
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_chown32 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall3 __NR_chown32, %1, %2, %3
%else
 %error "Function `sys_chown32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setuid32
;;;  calls sys_setuid -> kernel/sys.c
;;;  man: setuid(2)
;;;
;;;  In: sys_setuid32 uid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setuid32 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall1 __NR_setuid32, %1
%else
 %error "Function `sys_setuid32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setgid32
;;;  calls sys_setgid -> kernel/sys.c
;;;  man: setgid(2)
;;;
;;;  In: sys_setgid32 gid
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_setgid32 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall1 __NR_setgid32, %1
%else
 %error "Function `sys_setgid32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setfsuid32
;;;  calls sys_setfsuid -> kernel/sys.c
;;;  man: setfsuid(2)
;;;
;;;  In: sys_setfsuid32 uid
;;;  Out: eax=prev. val. of fsuid (on success); cur. val. of fsuid (on error)
;;;
%macro sys_setfsuid32 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall1 __NR_setfsuid32, %1
%else
 %error "Function `sys_setfsuid32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; setfsgid32
;;;  calls sys_setfsgid -> kernel/sys.c
;;;  man: setfsgid(2)
;;;
;;;  In: sys_setfsgid32 gid
;;;  Out: eax=prev. val. of fsgid (on success); cur. val. of fsgid (on error)
;;;
%macro sys_setfsgid32 1
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall1 __NR_setfsgid32, %1
%else
 %error "Function `sys_setfsgid32' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; pivot_root
;;;  calls sys_pivot_root -> fs/super.c
;;;  man: pivot_root(2)
;;;
;;;  In: sys_pivot_root *new_root, *put_old
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_pivot_root 2
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,3,41)
	__syscall2 __NR_pivot_root, %1, %2
%else
 %error "Function `sys_pivot_root' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; mincore
;;;  calls sys_mincore -> mm/filemap.c
;;;  man: mm/filemap.c
;;;
;;;  In: sys_mincore start, len, *vec
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_mincore 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall3 __NR_mincore, %1, %2, %3
%else
 %error "Function `sys_mincore' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; madvise
;;;  calls sys_madvise -> mm/filemap.c
;;;  man: mm/filemap.c
;;;
;;;  In: sys_madvise start, len, behavior
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_madvise 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall3 __NR_madvise, %1, %2, %3
%else
 %error "Function `sys_madvise' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; madvise1
;;;  calls sys_madvise -> mm/filemap.c
;;;  man: mm/filemap.c
;;;
;;;  In: sys_madvise1 start, len, behavior
;;;  Out: eax=0 (on success); -1 (on error)
;;;
%macro sys_madvise1 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall3 __NR_madvise1, %1, %2, %3
%else
 %error "Function `sys_madvise1' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; getdents64
;;;  calls sys_getdents64 -> fs/readdir.c
;;;  man: getdents(2)
;;;
;;;  In: sys_getdents64 fd, *dirent, count
;;;  Out: eax=bytes read (on success); -1 (on error)
;;;
%macro sys_getdents64 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall3 __NR_getdents64, %1, %2, %3
%else
 %error "Function `sys_getdents64' may not be supported by selected kernel version."
%endif
%endm


;;;
;;; fcntl64
;;;  calls sys_fcntl64 -> fs/fcntl.c
;;;  man: fcntl(2)
;;;
;;;  In: sys_fcntl64 fd, cmd, arg
;;;  Out: eax=see fcntl(2) (on success); -1 (on error)
;;;
%macro sys_fcntl64 3
%if ___KERNEL_VER___ >= ___CREATE_VER___(2,4,0)
	__syscall3 __NR_fcntl64, %1, %2, %3
%else
 %error "Function `sys_fcntl64' may not be supported by selected kernel version."
%endif
%endm

%endif	; _system_h_



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: