;;;
;;; Writef.s
;;;
;;; created: <Wed May 26 14:46:22 1999>
;;; last modified: <Sun Apr 23 21:19:36 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

;;;
;;; Writes buf to fd. buf is ASCIIZ.
;;; '%%' = '%', '%a'=INT, '%b'=8 bit HEX, '%c'=16 bit HEX, '%d'=32 bit HEX
;;; '%e'=*char, '%f'=signed INT, '%gxx' Float with precision xx, '%hxx' Float
;;; with precision xx scientific output (xEy = x*10^y), '%ixx' Float with
;;; precision xx scientific output (xey = x*e^y)
;;; '\n'=lf, '\\'='\', '\t'=ht, '\r'=cr, '\a'=bel
;;; '\b'=bs, '\0'=null, '\v'=vt, '\f'=ff
;;; In: Writef fd, *buf, *args
;;; Out: eax=0 (on success); eax=-1 (on error)
;;;
;;; buf: db "TEXT",0
;;; args: dd 1. param, 2.param
;;;

%include "system.h"
%include "string.h"
%include "io.h"

global _Writef

extern _errno,_IOpercent,_IOminus,_IObackslash,_IOnul,_IObel,_IObs,_IOht,_IOlf
extern _IOvt,_IOff,_IOcr


%define _fd ebx+0
%define buf ebx+4
%define args ebx+8

%define fd ebp-4
%define pos1 ebp-8
%define pos2 ebp-12
%define arg_pos ebp-16

	section .text
	align 32
_Writef:
	push ebp
	mov ebp,esp
	sub esp,byte 16

	__push ebx, ecx, edx, edi

	mov ebx,eax
	mov eax,[_fd]
	mov [fd],eax

	sys_flock dword [fd], dword LOCK_EX
	cmp eax,0
	jl near .err
	mov eax,[args]
	mov [arg_pos],eax

	mov eax,[buf]
	mov [pos1],eax
.loop0:
	mov ebx,[pos1]
	align 32
.loop:
	cmp [ebx],byte 0
	je near .nearlyexit
	cmp [ebx],byte '%'
	je near .work_on_param
	cmp [ebx],byte '\'
	je .maybe_crlf
	inc ebx
	jmp short .loop

.maybe_crlf:
	cmp [ebx+1],byte 'n'
	je near .lf
	cmp [ebx+1],byte '\'
	je near .backslash
	cmp [ebx+1],byte 't'
	je near .ht
	cmp [ebx+1],byte '0'
	je near .nul
	cmp [ebx+1],byte 'b'
	je near .bs
	cmp [ebx+1],byte 'r'
	je near .cr
	cmp [ebx+1],byte 'a'
	je near .bel
	cmp [ebx+1],byte 'v'
	je near .vt
	cmp [ebx+1],byte 'f'
	je near .ff
	inc ebx			; do not know... assume '\\', so jmp to .loop
	jmp short .loop

	align 32
.work_on_param:
	mov [pos2],ebx
	call .write
	cmp eax,0
	jl near .err

	mov eax,[pos2]
	add eax,byte 2
	mov [pos1],eax

	inc ebx
	movzx ecx,byte [ebx]
	cmp cl,'%'
	jne .nopercent
	sys_write dword [fd], dword _IOpercent, dword 1
	cmp eax,0
	jl near .err
	jmp .loop0

	align 32
.nopercent:
	sub cl,96
	jz near .err2
	dec ecx
	jnz .parameterb
	;; parameter a
	mov eax,[arg_pos]
	WriteInt dword [fd], dword [eax]
	add [arg_pos],dword 4
	jmp .loop

	align 32
.parameterb:
	dec ecx
	jnz .parameterc
	;; parameter b
	mov eax,[arg_pos]
	WriteHex dword [fd], dword [eax], dword 8
	add [arg_pos],dword 4
	jmp .loop

	align 32
.parameterc:
	dec ecx
	jnz .parameterd
	;; parameter c
	mov eax,[arg_pos]
	WriteHex dword [fd], dword [eax], dword 16
	add [arg_pos],dword 4
	jmp .loop

	align 32
.parameterd:
	dec ecx
	jnz .parametere
	;; parameter d
	mov eax,[arg_pos]
	WriteHex dword [fd], dword [eax], dword 32
	add [arg_pos],dword 4
	jmp .loop

	align 32
.parametere:
	dec ecx
	jnz .parameterf
	;; parameter e
	mov eax,[arg_pos]
	mov eax,[eax]
	WriteString dword [fd], eax
	add [arg_pos],dword 4
	jmp .loop

	align 32
.parameterf:
	dec ecx
	jnz near .parameterg
	;; parameter f
	mov eax,[arg_pos]
	mov eax,[eax]
	cmp eax,dword 0
	jnl .parameterf_j1
	sys_write dword [fd], dword _IOminus, dword 1
	cmp eax,0
	jl near .err
	mov eax,[arg_pos]
	mov eax,[eax]
	neg eax
.parameterf_j1:
	WriteInt dword [fd], eax
	add [arg_pos],dword 4
	jmp .loop

	align 32
.parameterg:
	dec ecx
	jnz near .parameterh
	;; parameter f
	movzx edx,byte [ebx+1]
	sub edx,byte 48
	imul edx,10
	movzx eax,byte [ebx+2]
	sub eax,byte 48
	add edx,eax
	add [pos1],dword 2

	mov eax,[arg_pos]
	WriteFloat dword [fd], dword [eax], edx
	add [arg_pos],dword 4
	jmp .loop

	align 32
.parameterh:
	dec ecx
	jnz near .parameteri
	;; parameter h
	movzx edx,byte [ebx+1]
	sub edx,byte 48
	imul edx,10
	movzx eax,byte [ebx+2]
	sub eax,byte 48
	add edx,eax
	add [pos1],dword 2

	mov eax,[arg_pos]
	WriteFloatScE dword [fd], dword [eax], edx
	add [arg_pos],dword 4
	jmp .loop

	align 32
.parameteri:
	dec ecx
	jnz near .err2
	;; parameter h
	movzx edx,byte [ebx+1]
	sub edx,byte 48
	imul edx,10
	movzx eax,byte [ebx+2]
	sub eax,byte 48
	add edx,eax
	add [pos1],dword 2

	mov eax,[arg_pos]
	WriteFloatSce dword [fd], dword [eax], edx
	add [arg_pos],dword 4
	jmp .loop

	align 32
.backslash:
	mov [pos2],ebx
	call .write
	cmp eax,0
	jl near .err
	mov eax,[pos2]
	add eax,byte 2
	mov [pos1],eax
	sys_write dword [fd], dword _IObackslash, dword 1
	cmp eax,0
	jl near .err
	jmp .loop0

	align 32
.lf:
	mov [pos2],ebx
	call .write
	cmp eax,0
	jl near .err

	mov eax,[pos2]
	add eax,byte 2
	mov [pos1],eax

	sys_write dword [fd], dword _IOlf, dword 1
	cmp eax,0
	jl near .err
	jmp .loop0

	align 32
.ht:
	mov [pos2],ebx
	call .write
	cmp eax,0
	jl near .err

	mov eax,[pos2]
	add eax,byte 2
	mov [pos1],eax

	sys_write dword [fd], dword _IOht, dword 1
	cmp eax,0
	jl near .err
	jmp .loop0

	align 32
.cr:
	mov [pos2],ebx
	call .write
	cmp eax,0
	jl near .err

	mov eax,[pos2]
	add eax,byte 2
	mov [pos1],eax

	sys_write dword [fd], dword _IOcr, dword 1
	cmp eax,0
	jl near .err
	jmp .loop0

	align 32
.nul:
	mov [pos2],ebx
	call .write
	cmp eax,0
	jl near .err

	mov eax,[pos2]
	add eax,byte 2
	mov [pos1],eax

	sys_write dword [fd], dword _IOnul, dword 1
	cmp eax,0
	jl near .err
	jmp .loop0

	align 32
.bs:
	mov [pos2],ebx
	call .write
	cmp eax,0
	jl near .err

	mov eax,[pos2]
	add eax,byte 2
	mov [pos1],eax

	sys_write dword [fd], dword _IObs, dword 1
	cmp eax,0
	jl near .err
	jmp .loop0

	align 32
.bel:
	mov [pos2],ebx
	call .write
	cmp eax,0
	jl near .err

	mov eax,[pos2]
	add eax,byte 2
	mov [pos1],eax

	sys_write dword [fd], dword _IObel, dword 1
	cmp eax,0
	jl near .err
	jmp .loop0

	align 32
.vt:
	mov [pos2],ebx
	call .write
	cmp eax,0
	jl near .err

	mov eax,[pos2]
	add eax,byte 2
	mov [pos1],eax

	sys_write dword [fd], dword _IOvt, dword 1
	cmp eax,0
	jl near .err
	jmp .loop0

	align 32
.ff:
	mov [pos2],ebx
	call .write
	cmp eax,0
	jl near .err

	mov eax,[pos2]
	add eax,byte 2
	mov [pos1],eax

	sys_write dword [fd], dword _IOff, dword 1
	cmp eax,0
	jl near .err
	jmp .loop0

	align 32
.nearlyexit:
	mov [pos2],ebx
	call .write

	push eax
	sys_flock dword [fd], dword LOCK_UN
	pop eax
.exit:
	__pop edi, edx, ecx, ebx

	leave
	ret

.err2:
	mov eax,-EINVAL
	jmp short .err

.err:
	mov [_errno],eax
	mov eax,-1
	jmp short .exit

	align 32
.write:
	mov eax,[pos2]
	sub eax,[pos1]
	sys_write dword [fd], dword [pos1], eax
	ret



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: