;;;
;;; MemoryFree.s
;;;
;;; created: <19.10.1998>
;;; last modified: <Wed Apr 26 17:21:14 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This is to handle memory allocation in assembly much easier.
;;;

;;; Sets up a memory managment system which uses chunks. If the user
;;; application destroys them, this system will fail and do sys_exit(1).

;;; If the application wants to free allocated memory which is mapped, it
;;; should unmap it first!

;;; The code is self-explaining :-)

%include "system.h"
%include "mem.h"
%include "io.h"

global _MemoryFree

extern _errno,_memory_brk,_memory_firstchunk,_memory_LOCK,_memory_HAS_INIT
extern _MemoryInit,_MemoryAllowReadWriteHeaders,_MemoryRestoreHeaders
extern _MemoryTune,_MemoryChunkStructureIsCorrupted

	section .text
	align 32

;;;
;;; MemoryFree
;;;
;;;  In: MemoryFree ptr
;;;  Out: eax=0 (on success); -1 (on error)
;;;
;;; +++ Achtung: _MemoryFree setzt nur das Flag auf free, Tune mu den Mll
;;;              beseitigen!
_MemoryFree:
	__push edx, esi, edi

	mov esi,eax

	MemoryHAS_INIT		; '_MemoryFree: '
	MemoryLOCK

	call _MemoryAllowReadWriteHeaders

; Verify Argument
	sub esi,_memory_chunk_size	; need header adress
	mov edi,[_memory_firstchunk]
	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jnz .NotValid

.VLoop:
; Check if ID is OK, if not, exit an give ERROR!
	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_flag_IDmask
	cmp edx,_memory_identify_flag
	je .VLoop_j1
	mov eax,dword .rout_name
	jmp _MemoryChunkStructureIsCorrupted
.VLoop_j1:

	add edi,[edi+_memory_chunk.size]
	cmp edi,esi
	je .TestFlagOK

	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jz .VLoop

; Sorry, this is not a valid Pointer
.NotValid:
	mov [_errno],dword -EINVAL
.NotValid2:
	call _MemoryRestoreHeaders
	MemoryUNLOCK
	__pop edi, esi, edx
	mov eax,-1
	ret

.TestFlagOK:
;;; Now check if chunk access is forbidden
	mov eax,[esi+_memory_chunk.flags]
	and eax,_memory_forbidmodify_flag
	jz .TestFlagOK2
;;; Is this the same job?
	sys_getpid
	cmp [esi+_memory_chunk.pid],eax
	je .TestFlagOK2
	mov [_errno],dword -EFAULT
	jmp short .NotValid2

.TestFlagOK2:
;;; If .proc is not NULL, do jmp [.proc]
	cmp [esi+_memory_chunk.proc],dword 0
	je .ReleaseIt
;;; Restore to original state, set eax to [ptr]
	call _MemoryRestoreHeaders
	MemoryUNLOCK
	mov eax,esi
	__pop edi, esi, edx
	add eax,_memory_chunk_size
	jmp near dword [eax-_memory_chunk_size+_memory_chunk.proc]

.ReleaseIt:
	mov eax,[esi+_memory_chunk.flags]
	or eax,_memory_freechunk_flag
	and eax,~(_memory_flag_MPROTmask|_memory_forbidmodify_flag)
	or eax,_memory_PROT_NONE
	mov [esi+_memory_chunk.flags],eax
	sys_mprotect esi, dword [esi+_memory_chunk.size], dword  PROT_NONE
	sys_mprotect esi, dword PAGE_SIZE, dword  PROT_READ|PROT_WRITE

	call _MemoryRestoreHeaders
	MemoryUNLOCK
	__pop edi, esi, edx
; Tune Memory!
	call _MemoryTune
	mov eax,0
	ret

	section .data
.rout_name:	db '_MemoryFree',0



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: