;;;
;;; MemoryGetsize.s
;;;
;;; created: <Thu Apr 22 18:42:28 1999>
;;; last modified: <Tue Apr  4 19:31:02 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This is to handle memory allocation in assembly much easier.
;;;

;;; Sets up a memory managment system which uses chunks. If the user application
;;; destroys them, this system will fail and do sys_exit(1).

;;; If the application wants to free allocated memory which is mapped, it should
;;; unmap it first!

;;; The code is self-explaining :-)

%include "system.h"
%include "mem.h"
%include "io.h"

global _MemoryGetsize

extern _errno,_memory_brk,_memory_firstchunk,_memory_LOCK,_memory_HAS_INIT
extern _MemoryInit,_MemoryAllowReadWriteHeaders,_MemoryRestoreHeaders
extern _MemoryTune,_MemoryChunkStructureIsCorrupted

	section .text
	align 32

;;;
;;; MemoryGetsize
;;;
;;;  In: MemoryGetsize ptr
;;;  Out: eax=Size (on success); -1 (on error)
;;;
_MemoryGetsize:
	__push edx, esi, edi

	mov esi,eax

	MemoryHAS_INIT
	MemoryLOCK

	call _MemoryAllowReadWriteHeaders

; Verify Argument
	sub esi,_memory_chunk_size	; need header adress
	mov edi,[_memory_firstchunk]
	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jnz .NotValid

.VLoop:
; Check if ID is OK, if not, exit an give ERROR!
	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_flag_IDmask
	cmp edx,_memory_identify_flag
	je .VLoop_j1
	mov eax,dword .rout_name
	jmp _MemoryChunkStructureIsCorrupted
.VLoop_j1:

	add edi,[edi+_memory_chunk.size]
	cmp edi,esi
	je .TestFlagOK

	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jz .VLoop

; Sorry, this is not a valid Pointer
.NotValid:
	mov [_errno],dword -EINVAL
	call _MemoryRestoreHeaders
	MemoryUNLOCK
	__pop edi, esi, edx
	mov eax,-1
	ret

.TestFlagOK:
	mov eax,[esi+_memory_chunk.size]
	sub eax,_memory_chunk_size	; Usermemory is chunk.size-chunk_size
	;; retval in eax

	call _MemoryRestoreHeaders
	MemoryUNLOCK
	__pop edi, esi, edx
	ret

	section .data
.rout_name:	db '_MemoryGetsize',0



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: