;;;
;;; GetCPUType.s
;;;
;;; created: <05.12.1998>
;;; last modified: <Thu Oct 24 23:35:38 2002>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

;;;
;;; Gets type of CPU an FPU
;;; In: _GetCPUType *CPUInfo
;;; Out: dword [CPUInfo.ProcessorType]
;;;		0 (386), 1 (486), 2 (586), ...
;;;	 dword [CPUInfo.CPUExtensions]
;;;		CPU Extensions
;;;			CPUInfo_HaveFPU: FPU
;;;			CPUInfo_HaveCPUID: cpuid supported
;;;			CPUInfo_HaveMMX: MMX
;;;			CPUInfo_Have3DNow: 3DNow!
;;;			CPUInfo_HaveCMPXCHG8B: cmpxchg8b supported
;;;			CPUInfo_HaveCondMOV: conditional move instructions
;;;			CPUInfo_Have3DNowExt: 3DNowExt
;;;			CPUInfo_HaveMMXExt; MMXExt
;;;	 4 dwords [CPUInfo.VendorString]
;;;		vendor string (12 bytes+4 bytes NULL)
;;;

%include "system.h"
%include "misc.h"

global _GetCPUType

	section .text
	align 32
_GetCPUType:
	pushfd			; If we don't do this, we'll get in trouble!!
	__push eax, ebx, ecx, edx, edi

	mov edi,eax

;;; CPU
;;; Out: 3=386,4=486,5=586,6=686,...(?)
;;; at least an 386
        mov [edi+CPUInfo.ProcessorType],dword 3
;;;  486?
	pushfd  ; Bit 18 test, if set -> 486
	pop eax
	mov ebx,eax
	xor eax,0x40000
	push eax
	popfd
	pushfd
	pop eax
	xor eax,ebx
	jz near .CPUEnd
;;; Ok, we've got an 486
	mov [edi+CPUInfo.ProcessorType],dword 4
;;; It may be a 586...
	pushfd		; If I can set bit 21, I've got CPUID
	pop eax		; an the exact type
	mov ebx,eax
	or eax,0x200000
	push eax
	popfd
	pushfd
	pop eax
	xor eax,ebx
;;; OK, only 486...
	jz near .CPUEnd
;;; Hey, we've got CPUID
	or [edi+CPUInfo.CPUExtensions],dword CPUInfo_HaveCPUID
	mov eax,1
	cpuid
	shr ax,8
	and eax,0x0f
	mov [edi+CPUInfo.ProcessorType],eax

;;; Copy vendor string
	mov eax,0
	cpuid
	mov [edi+CPUInfo.VendorString+0],ebx
	mov [edi+CPUInfo.VendorString+4],edx
	mov [edi+CPUInfo.VendorString+8],ecx

.Intel:
;;; Test for cmpxchg8b support
	bt edx,8
	jnc .noCMPXCHG8B	; No cmpxchg8b support
	or [edi+CPUInfo.CPUExtensions],dword CPUInfo_HaveCMPXCHG8B
.noCMPXCHG8B:

;;; Test for conditional move instructions
	bt edx,15
	jnc .noCondMOV	; No conditional move instructions
	or [edi+CPUInfo.CPUExtensions],dword CPUInfo_HaveCondMOV
.noCondMOV:

;;; Test for MMX support
	mov eax,1
	cpuid
	bt edx,23
	jnc .noMMX	; No MMX support
	or [edi+CPUInfo.CPUExtensions],dword CPUInfo_HaveMMX
.noMMX:

;;; extended functions supported?
	mov eax,0x80000000
	cpuid
	cmp eax,0x80000000
	jbe .CPUEnd	; No extended functions

;;; Need to test for Vendorstring=AutenticAMD?
;;; test for 3DNow!
	mov eax,0x80000001
	cpuid
	bt edx,31
	jnc .CPUEnd
	or [edi+CPUInfo.CPUExtensions],dword CPUInfo_Have3DNow

;;; test for 3DNowExt
	mov eax,0x80000001
	cpuid
	bt edx,30
	jnc .CPUEnd
	or [edi+CPUInfo.CPUExtensions],dword CPUInfo_Have3DNowExt

;;; test for mmxext
	mov eax,0x80000001
	cpuid
	bt edx,22
	jnc .CPUEnd
	or [edi+CPUInfo.CPUExtensions],dword CPUInfo_HaveMMXExt

.CPUEnd:
; Get FPU
	fninit			; reset FP status word
	mov ax,0xcfcf		; initialize temp word to non-zero value
	fnstsw ax		; save FP status word
				; check FP status word
	cmp al,0		; see if correct status with
				; written
	jne .End		; no fpu present
; FPU present
	or [edi+CPUInfo.CPUExtensions],dword CPUInfo_HaveFPU

.End:
	__pop edi, edx, ecx, ebx, eax
	popfd
	ret


;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: