;;;
;;; XCreateGC.s
;;;
;;; created: <Sat May 27 20:00:48 2000>
;;; last modified: <Fri Jun  8 19:35:19 2001>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

%include "system.h"
%include "x_func.h"
%include "x_def.h"
%include "x_struc_x.h"
%include "mem.h"

global _XCreateGC

%define dpy ebx+0
%define wid ebx+4
%define valuemask ebx+8
%define values ebx+12

%define request ecx

%macro vl 4			; Flag, Value, Std. val., (1|2|4)
;;; Set val only if flag is set
	mov eax,[request+12]
	and eax,%1
	jz short %%end
;;; Set val only if it differs from std
	mov eax,%2
	cmp eax,%3
	je %%isstd
%if %4 = 1	; byte
	mov [ebp],al
	inc ebp
	inc word [request+2]
%elif %4 = 2	; word
	mov [ebp],ax
	inc ebp
	inc ebp
	add [request+2],word 2
%elif %4 = 4	; 4
	mov [ebp],eax
	add ebp,byte 4
	add [request+2],word 4
%else
 %error "Watch your code!"
%endif
	jmp short %%end
%%isstd:
;;; unset flag: val is std
	and [request+12],dword ~(%1) ; Unset the flag
%%end:
%endm

	section .text
	align 32
_XCreateGC:
	__push ebx, ecx, edx, ebp
	mov ebx,eax
	MemoryAllocate dword 4*27
	cmp eax,-1
	je near .end
	mov request,eax

;;; Opcode: CreateGC
	mov [request+0],byte X_CreateGC
;;; request length (later modified, shr 2)
	mov [request+2],word 4*4
;;; cid
	XAllocID dword [dpy]
	mov [request+4],eax
;;; wid
	mov eax,[wid]
	mov [request+8],eax
;;; valuemask
	mov eax,[valuemask]
	and eax,(1<<(GCLastBit+1))-1
	mov [request+12],eax
;;; values
	mov edx,[values]
	lea ebp,[request+16]

	vl GCFunction, [edx+XGCValues.function], GXcopy, 4 ; 1
	vl GCPlaneMask, [edx+XGCValues.plane_mask], AllPlanes, 4
	vl GCForeground, [edx+XGCValues.foreground], 0, 4
	vl GCBackground, [edx+XGCValues.background], 1, 4
	vl GCLineWidth, [edx+XGCValues.line_width], 0, 4 ; 2
	vl GCLineStyle, [edx+XGCValues.line_style], LineSolid, 4 ; 1
	vl GCCapStyle, [edx+XGCValues.cap_style], CapButt, 4 ; 1
	vl GCJoinStyle, [edx+XGCValues.join_style], JoinMiter, 4 ; 1
	vl GCFillStyle, [edx+XGCValues.fill_style], FillSolid, 4 ; 1
	vl GCFillRule, [edx+XGCValues.fill_rule], EvenOddRule, 4 ; 1
	vl GCTile, [edx+XGCValues.tile], ((~0)&(~(111b<<(32-3)))), 4
	vl GCStipple, [edx+XGCValues.stipple], ((~0)&(~(111b<<(32-3)))), 4
	vl GCTileStipXOrigin, [edx+XGCValues.ts_x_origin], 0, 4	; 2
	vl GCTileStipYOrigin, [edx+XGCValues.ts_y_origin], 0, 4	; 2
	vl GCFont, [edx+XGCValues.font], ((~0)&(~(111b<<(32-3)))), 4
	vl GCSubwindowMode, [edx+XGCValues.subwindow_mode], ClipByChildren, 4 ; 1
	vl GCGraphicsExposures, [edx+XGCValues.graphics_exposures], xTrue, 4 ; 1
	vl GCClipXOrigin, [edx+XGCValues.clip_x_origin], 0, 4 ; 2
	vl GCClipYOrigin, [edx+XGCValues.clip_y_origin], 0, 4 ; 2
	vl GCClipMask, [edx+XGCValues.clip_mask], None, 4
	vl GCDashOffset, [edx+XGCValues.dash_offset], 0, 4 ; 2
	vl GCDashList, [edx+XGCValues.dashes], 4, 4 ; 1
	vl GCArcMode, [edx+XGCValues.arc_mode], ArcPieSlice, 4 ; 1

;;; pad request
	mov ax,[request+2]
	and ax,3
	jz .pad_fettich
	neg ax
	add eax,byte 4
	mov [ebp],byte 0
	inc ebp
	dec ax
	jz .pad_fettich
	mov [ebp],byte 0
	inc ebp
	dec ax
	jz .pad_fettich
	mov [ebp],byte 0

.pad_fettich:
	movzx ebp,word [request+2]
	add ebp,byte 3
	and ebp,~3
	mov eax,ebp
	shr eax,2
	mov [request+2],ax
	XSync dword [dpy], request, ebp

;;; Return value
	mov eax,[request+4]

	push eax
	MemoryFree request
	cmp eax,-1
	pop eax
	jne near .end
	mov eax,-1
.end:
	__pop ebp, edx, ecx, ebx
	ret


;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: