;;;
;;; XCreateWindow.s
;;;
;;; created: <Tue Dec 14 16:04:58 1999>
;;; last modified: <Fri Jun  8 19:08:54 2001>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

%include "system.h"
%include "x.h"
%include "mem.h"

global _XCreateWindow

%define dpy edx+0
%define parent edx+4
%define x edx+8
%define y edx+12
%define width edx+16
%define height edx+20
%define borderWidth edx+24
%define depth edx+28
%define class edx+32
%define visual edx+36
%define valuemask edx+40
%define attributes edx+44

%define request ebp

%macro vl 3			; Flag, Value, (1|4)
	mov eax,[request+28]
	and eax,%1
	jz %%end
	mov eax,%2
%if %3 = 1	; byte
	mov [ebx],al
	inc ebx
	inc word [request+2]
%elif %3 = 4	; dword
	mov [ebx],eax
	add ebx,byte 4
	add [request+2],word 4
%else
 %error "Watch your code!"
%endif
%%end:
%endm

	section .text
	align 32
_XCreateWindow:
	__push ebx, ecx, edx, ebp
	mov edx,eax

;;; Get memory for request
	MemoryAllocate dword 32+32*4
	cmp eax,-1
	je near .end
	mov request,eax

;;; Opcode: CreateWindow
	mov [request+0],byte X_CreateWindow
;;; Depth
	mov eax,[depth]
	mov [request+1],al
;;; request length (later modified, shr 2)
	mov [request+2],word 8*4
;;; wid
	XAllocID dword [dpy]
	mov [request+4],eax
;;; parent
	mov eax,[parent]
	mov [request+8],eax
;;; x
	mov eax,[x]
	mov [request+12],ax
;;; y
	mov eax,[y]
	mov [request+14],ax
;;; width
	mov eax,[width]
	mov [request+16],ax
;;; height
	mov eax,[height]
	mov [request+18],ax
;;; border-width
	mov eax,[borderWidth]
	mov [request+20],ax
;;; class
	mov eax,[class]
	mov [request+22],ax
;;; visual
	mov eax,[visual]
	cmp eax,CopyFromParent
	jne .visual_j1
	mov [request+24],dword CopyFromParent
	jmp short .visual_j2
.visual_j1:
	mov eax,[dpy]
	mov eax,[eax+Visual.visualid]
	mov [request+24],eax
.visual_j2:

;;; valuemask
	mov eax,[valuemask]
	and eax,AllMaskBits
	mov [request+28],eax

;;; Value-list
	lea ebx,[request+32]
	mov ecx,[attributes]

	vl CWBackPixmap, [ecx+XSetWindowAttributes.background_pixmap], 4
	vl CWBackPixel, [ecx+XSetWindowAttributes.background_pixel], 4
	vl CWBorderPixmap, [ecx+XSetWindowAttributes.border_pixmap], 4
	vl CWBorderPixel, [ecx+XSetWindowAttributes.border_pixel], 4
	vl CWBitGravity, [ecx+XSetWindowAttributes.bit_gravity], 4 ; 1
	vl CWWinGravity, [ecx+XSetWindowAttributes.win_gravity], 4 ; 1
	vl CWBackingStore, [ecx+XSetWindowAttributes.backing_store], 4 ; 1
	vl CWBackingPlanes, [ecx+XSetWindowAttributes.backing_planes], 4
	vl CWBackingPixel, [ecx+XSetWindowAttributes.backing_pixel], 4
	vl CWOverrideRedirect, [ecx+XSetWindowAttributes.override_redirect], 4 ; 1
	vl CWSaveUnder, [ecx+XSetWindowAttributes.save_under], 4 ; 1
	vl CWEventMask, [ecx+XSetWindowAttributes.event_mask], 4
	vl CWDontPropagate, [ecx+XSetWindowAttributes.do_not_propagate_mask], 4
	vl CWColormap, [ecx+XSetWindowAttributes.colormap], 4
	vl CWCursor, [ecx+XSetWindowAttributes.cursor], 4

;;; pad request
	mov ax,[request+2]
	and ax,3
	jz .pad_fettich
	neg ax
	add eax,byte 4
	mov [ebx],byte 0
	inc ebx
	dec ax
	jz .pad_fettich
	mov [ebx],byte 0
	inc ebx
	dec ax
	jz .pad_fettich
	mov [ebx],byte 0

.pad_fettich:
	movzx ecx,word [request+2]
	add ecx,byte 3
	and ecx,~3
	mov eax,ecx
	shr eax,2
	mov [request+2],ax
	XSync dword [dpy], request, ecx

	mov eax,[request+4]

;;; Free memory block
	push eax
	MemoryFree request
	cmp eax,-1
	pop eax
	jne .end
	mov eax,-1
.end:
	__pop ebp, edx, ecx, ebx
	ret

;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: