;;;
;;; XShmCreateImage.s
;;;
;;; created: <Sun Aug 20 14:42:54 2000>
;;; last modified: <Sat Sep  9 21:01:45 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

%include "system.h"
%include "x_func.h"
%include "x_def.h"
%include "x_struc.h"
%include "mem.h"
%include "string.h"
%include "sys.h"

global _XShmCreateImage

extern _errno

%define dpy ebx+0
%define image ebx+4
%define visual ebx+8
%define depth ebx+12
%define format ebx+16
%define data ebx+20
%define shminfo ebx+24
%define width ebx+28
%define height ebx+32

%define request ecx
%define reply ecx

	section .text
	align 32
_XShmCreateImage:
	__push ebx, ecx, edx, esi
	mov ebx,eax

;;; MIT-SHM Extension availible?
	mov eax,[dpy]
	cmp [eax+dpystruc.MIT_SHM_opcode],dword 0
	je near .notavail		; No :(

;;; If image = 0, allocate the buffer for it. Otherwise, the caller has already
;;; defined one.
	mov esi,[image]
	cmp esi,dword 0
	jne .hasimage
	MemoryAllocate dword ximagestruc_size
	cmp eax,-1
	je near .end
	mov esi,eax
.hasimage:

	mov eax,[data]
	mov [esi+ximagestruc.data],eax

	mov eax,[shminfo]
	mov [esi+ximagestruc.obdata],eax

	mov eax,[width]
	mov [esi+ximagestruc.width],eax

	mov eax,[height]
	mov [esi+ximagestruc.height],eax

	mov eax,[depth]
	mov [esi+ximagestruc.depth],eax

	mov ecx,[dpy]
	mov eax,[ecx+dpystruc.bitmapBitOrder]
	mov [esi+ximagestruc.bitmap_bit_order],eax

	mov eax,[ecx+dpystruc.bitmapScanlineUnit]
	mov [esi+ximagestruc.bitmap_unit],eax

	XGetScanlinePad ecx, dword [depth]
	mov [esi+ximagestruc.bitmap_pad],eax

	mov [esi+ximagestruc.xoffset],dword 0

	cmp [visual],dword 0
	je .visual_null

	mov ecx,[visual]
	mov eax,[ecx+Visual.red_mask]
	mov [esi+ximagestruc.red_mask],eax

	mov eax,[ecx+Visual.green_mask]
	mov [esi+ximagestruc.green_mask],eax

	mov eax,[ecx+Visual.blue_mask]
	mov [esi+ximagestruc.blue_mask],eax
	jmp short .visual_ok

.visual_null:
	mov [esi+ximagestruc.red_mask],dword 0
	mov [esi+ximagestruc.green_mask],dword 0
	mov [esi+ximagestruc.blue_mask],dword 0

.visual_ok:

	cmp [format],dword ZPixmap
	jne .zpixmap1
	XGetBitsPerPixel dword [dpy], dword [depth]
	mov [esi+ximagestruc.bits_per_pixel],eax
	jmp .zpixmapok
.zpixmap1:
	mov [esi+ximagestruc.bits_per_pixel],dword 1
.zpixmapok:

	mov ecx,[width]
	mov eax,[esi+ximagestruc.bits_per_pixel]
	imul eax,ecx
	mov ecx,[esi+ximagestruc.bitmap_pad]
	shr ecx,3
	add eax,ecx
	dec eax
	imul ecx,ecx
	mov edx,0
	div ecx
	mov [esi+ximagestruc.bytes_per_line],eax

	mov eax,esi
.end:
	__pop esi, edx, ecx, ebx
	ret

.notavail:
	mov [_errno],dword -ENOPROTOOPT
	mov eax,-1
	jmp short .end

;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: