;;;
;;; UnzipLZW.s
;;;
;;; created: <Wed Mar 29 16:40:57 2000>
;;; last modified: <Wed Apr  5 23:18:52 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

;;;
;;; Uncompress buf_in
;;; In: _ZipLZW *buf_in (eax)
;;; Out: eax=buf_out(on success); eax=-1(on error)
;;;
;;; buf_in:	1 dword orig. size
;;;		1 dword compr. size (n)
;;;		1 dword crc32 (uncompressed data)
;;;		n bytes compr. data
;;;

%include "system.h"
%include "mem.h"
%include "string.h"
%include "io.h"

global _UnzipLZW

extern _errno

%define buf_out ebp-4
%define alph ebp-8
%define _crc ebp-12
%define dest_len ebp-16
%define	src_len ebp-20
%define src_pos ebp-24
%define decodebuf ebp-28
%define max_eintraege ebp-32
%define get_logical_byte_bits_count ebp-36
%define get_logical_byte_width ebp-40
%define get_logical_byte_restbyte ebp-44
%define old_byte ebp-48
%define sonder_byte ebp-52
%define temp_byte ebp-56

	section .text
	align 32
_UnzipLZW:
	push ebp
	mov ebp,esp
	sub esp,56
	__push ebx, ecx, edx, esi, edi

	mov ebx,eax
	mov eax,[ebx+0]
	mov [dest_len],eax
	mov eax,[ebx+4]
	mov [src_len],eax
	mov eax,[ebx+8]
	mov [_crc],eax
	add ebx,byte 12
	mov [src_pos],ebx

;;; Allocate buf_out
	MemoryAllocate dword [dest_len]
	cmp eax,-1
	je near .exit
	mov [buf_out],eax

;;; Allocate alph
	MemoryAllocate dword 4096*4
	cmp eax,-1
	je near .err1
	mov [alph],eax

;;; Allocate decode buf
	MemoryAllocate dword 4096
	cmp eax,-1
	je near .err0
	mov [decodebuf],eax

;;; Uncompress
	cld
	mov edx,258     ; Alphend
	mov edi,[buf_out]
	mov [max_eintraege],dword 511
	mov [get_logical_byte_bits_count],dword 0
	mov [get_logical_byte_width],dword 9
	mov [get_logical_byte_restbyte],dword 0
	call get_logical_byte
	stosb
	mov [old_byte],eax
	mov [sonder_byte],eax
	align 32
.main:
	call get_logical_byte
	cmp eax,257      ; Ende?
	je near .exit0
	cmp eax,256      ; Clr Alphabet?
	jne .main1
; Rcksetzen und Alphabet vorbereiten
	mov [max_eintraege],dword 511
	mov [get_logical_byte_width],dword 9
	mov edx,258
	call get_logical_byte
	stosb
	mov [old_byte],eax
	mov [sonder_byte],eax
	jmp short .main
.main1:
; Byte dekodieren...
	mov ecx,[alph]
	mov esi,[decodebuf]
	add esi,4096
	mov [temp_byte],eax
	cmp eax,edx
	jb .main3
; Sonderfall vorbereiten...
	mov eax,[old_byte]
	mov ebx,[sonder_byte]
	dec esi
	mov [esi],bl
.main3:
	cmp eax,255
	jbe .main4
	align 32
.main3_5:
; Byte auf Stack dekodieren
	mov bl,[ecx+eax*4+2]
	dec esi
	mov [esi],bl
	movzx eax,word [ecx+eax*4+0]
	cmp eax,257
	ja .main3_5
.main4:
; Byte ist dekodiert.
	dec esi
	mov [esi],al
	mov ebx,[old_byte]
	mov [ecx+edx*4+0],bx    ; Alphabeteintrag
	mov [ecx+edx*4+2],ax
	inc edx
	call check_bit_width
	mov [sonder_byte],eax
	mov ebx,[temp_byte]
	mov [old_byte],ebx
; Stack in Ziel schreiben
	mov ecx,[decodebuf]
	add ecx,4096
	sub ecx,esi
	Strncpy esi, edi, ecx
	add edi,ecx
	jmp .main

.exit0:
	MemoryFree dword [alph]
	MemoryFree dword [decodebuf]

	Crc32 dword [buf_out], dword [dest_len]
	cmp eax,dword [_crc]
	je .exit1
	mov eax,-1
	mov [_errno],dword -EPROTO ; ?
	jmp short .exit
.exit1:
	mov eax,[buf_out]

.exit:
	__pop edi, esi, edx, ecx, ebx
	leave
	ret



.err0:
	MemoryFree dword [alph]
.err1:
	MemoryFree dword [buf_out]
	mov eax,-1
	jmp short .exit


;;;
;;; Get logical byte
;;;
	align 32
get_logical_byte:
	__push ebx, esi
	mov esi,[src_pos]
	mov ecx,[get_logical_byte_bits_count]
	mov eax,[get_logical_byte_restbyte]
.j1:
	movzx ebx,byte [esi]
	inc esi
	shl ebx,cl
	or eax,ebx
	add ecx,byte 8
	cmp ecx,[get_logical_byte_width]
	jb .j1
	mov [src_pos],esi
	sub ecx,[get_logical_byte_width]
	mov [get_logical_byte_bits_count],ecx
	mov ecx,[get_logical_byte_width]
	mov ebx,eax
	shr ebx,cl
	mov [get_logical_byte_restbyte],ebx
	and eax,[max_eintraege]
	__pop esi, ebx
	ret

;;;
;;; Check bit width
;;;
	align 32
check_bit_width:
	cmp edx,[max_eintraege]
	ja .j1
	ret
	align 32
.j1:
	push ebx
	mov ecx,[get_logical_byte_width]
	inc ecx
	mov [get_logical_byte_width],ecx
	mov ebx,1
	shl ebx,cl
	dec ebx
	mov [max_eintraege],ebx
	pop ebx
	ret

;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: