;;;
;;; ZipLZW.s
;;;
;;; created: <Tue Mar 28 18:15:42 2000>
;;; last modified: <Thu Jun  1 17:11:12 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

;;;
;;; Compress buf_in
;;; In: _ZipLZW *buf_in, len
;;; Out: eax=buf_out(on success); eax=-1(on error)
;;;
;;; buf_out:	1 dword orig. size
;;;		1 dword compr. size (n)
;;;		1 dword crc32 (uncompressed data)
;;;		n bytes compr. data
;;;

%include "system.h"
%include "mem.h"
%include "string.h"

%ifdef ___VERBOSE_FLAG___
 %include "io.h"
%endif ; ___VERBOSE_FLAG___

global _ZipLZW

extern _errno

%define _buf_in ebx+0
%define _len ebx+4

%define buf_in ebp-4
%define len ebp-8

%define buf_out ebp-12		; output buffer
%define alph ebp-16		; buffer for alphabet

%define src_end ebp-20
%define dest_pos ebp-24		; actual pos of dest
%define dest_len ebp-28		; current dest length

%define write_logical_byte_width ebp-32
%define write_logical_byte_count ebp-36	; how many bits to write (0)
%define write_logical_byte_bits ebp-40	; rest of that bits (0)
%define alphabet_pos ebp-44	; pos/4 rel [alph]
%define max_eintraege ebp-48	; max alphabet width
%define _crc ebp-52

%ifdef ___VERBOSE_FLAG___
 %define percentbefore ebp-56
%endif ; ___VERBOSE_FLAG___

	section .text
	align 32
_ZipLZW:
	push ebp
	mov ebp,esp

%ifdef ___VERBOSE_FLAG___
	sub esp,56
	mov [percentbefore],dword 0
%else
	sub esp,52
%endif ; ___VERBOSE_FLAG___

	__push ebx, ecx, edx, esi, edi
	mov ebx,eax
	mov eax,[_buf_in]
	mov [buf_in],eax
	mov eax,[_len]
	mov [len],eax

;;; Calc crc32
	Crc32 dword [buf_in], dword [len]
	mov [_crc],eax

;;; Get memory for buf_out (should be smaller than len :)
	mov eax,[len]
	add eax,byte 12
	MemoryAllocate eax
	cmp eax,-1
	je near .exit
	mov [buf_out],eax

;;; Get memory for alphabet
	MemoryAllocate dword 4096*4
	cmp eax,-1
	je near .err0
	mov [alph],eax

;;; Init vars
	mov [write_logical_byte_width],dword 9
	mov [alphabet_pos],dword 258
	mov [write_logical_byte_count],dword 0
	mov [write_logical_byte_bits],dword 0
	cld
	mov [max_eintraege],dword 511

	mov eax,[buf_in]
	mov edi,eax		; src_pos
	add eax,[len]
	mov [src_end],eax
	mov eax,[buf_out]
	add eax,byte 12		; space for header
	mov [dest_pos],eax
	mov [dest_len],dword 0

	align 32
.main:
; Alphabetberlauf verhindern...
	cmp [alphabet_pos],dword 4096
	jb .main0
; ClrAlphabet-Zeichen senden
	mov eax,256
	call write_logical_byte
; Alphabet zurck
	mov [write_logical_byte_width],dword 9
	mov [alphabet_pos],dword 258
	mov [max_eintraege],dword 511
.main0:
	mov esi,257		; alphabet_pos_tmp
	cmp edi,[src_end]
	jb .main1
; Ende erreicht
	mov eax,257
	call write_logical_byte
	mov eax,0		; clear [write_logical_byte_bits]
	call write_logical_byte
	jmp .exit0

	align 32
.main1:
; NEIN, SUCHEN!!!!
	mov ebx,1
	movzx ecx,byte [edi+0]
	mov edx,[alph]
.main3:
	call search_entry
	jc short .main4
	mov al,[edi+ebx]
	cmp al,[edx+esi*4+2]
	jne short .main3
	mov ecx,esi
	inc ebx
	jmp short .main3

	align 32
.main4:
	mov eax,ecx
	call write_logical_byte
	movzx edx,byte [edi+ebx]
	call new_alphabetentry
	add edi,ebx
	jmp .main


.exit0:
	MemoryFree dword [alph]
	mov eax,[buf_out]
	mov ebx,[len]
	mov [eax+0],ebx
	mov ebx,[dest_len]
	mov [eax+4],ebx
	mov ebx,[_crc]
	mov [eax+8],ebx
	MemoryResize eax, dword [dest_len]

%ifdef ___VERBOSE_FLAG___
;;; Show status
	push eax
	mov eax,[dest_len]
	mov edx,100
	mul edx
	div dword [len]
	mov ebx,eax
	mov eax,10
	mov eax,[dest_len]
	mov edx,1000
	mul edx
	div dword [len]
	mov ecx,eax
	mov eax,ebx
	mov edx,10
	mul edx
	sub ecx,eax
	mov eax,edi
	sub eax,[buf_in]
	WritefSTDOUT cr,'compressed: %a/%a -> %a (%a.%a%%)  ',lf, eax, dword [len], dword [dest_len], ebx, ecx
	pop eax
%endif ; ___VERBOSE_FLAG___

.exit:
	__pop edi, esi, edx, ecx, ebx
	leave
	ret


.err0:
	MemoryFree dword [buf_out]
	mov eax,-1
	jmp short .exit

;;;
;;; Check alphabet width
;;;
	align 32
check_bit_width:
;;; inc width?
	mov eax,[alphabet_pos]
	cmp eax,[max_eintraege]
	ja .j1
	ret
	align 32
.j1:
;;; inc width!
	__push ebx, ecx
	mov ecx,[write_logical_byte_width]
	inc ecx
	mov ebx,1
	shl ebx,cl
	dec ebx
	mov [max_eintraege],ebx
	mov [write_logical_byte_width],ecx
	__pop ecx, ebx
	ret

;;;
;;; Write logical byte
;;;
;;; ax=logical byte
	align 32
write_logical_byte:
	__push ebx, ecx, edi

%ifdef ___VERBOSE_FLAG___
;;; Show status
	__push eax, edx
;;; Show only every percent!
	mov eax,edi
	sub eax,[buf_in]
	mov edx,100
	mul edx
	div dword [len]
	cmp eax,[percentbefore]
	je .dontshow
	mov [percentbefore],eax
	mov eax,edi
	sub eax,[buf_in]
	WritefSTDOUT cr,'compressing: %a/%a -> %a', eax, dword [len], dword [dest_len]
.dontshow:
	__pop edx, eax
%endif ; ___VERBOSE_FLAG___

	mov ecx,[write_logical_byte_count]
	shl eax,cl
	or eax,[write_logical_byte_bits]
	add ecx,[write_logical_byte_width]
	mov edi,[dest_pos]
	mov ebx,[dest_len]
.j1:
	cmp ecx,8
	jb .j3
	mov [edi],al
	inc ebx
	inc edi
	cmp ebx,[len]
	jb .j2
;;; exit here!
	__pop edi, ecx, ebx
	mov [dest_len],ebx
	add esp,byte 4		; cancel call
	MemoryFree dword [alph]
	mov [_errno],dword -ENOMEM
	jmp _ZipLZW.err0

	align 32
.j2:
	shr eax,8
	sub ecx,byte 8
	jnz short .j1
	align 32
.j3:
	mov [dest_pos],edi
	mov [dest_len],ebx
	mov [write_logical_byte_bits],eax
	mov [write_logical_byte_count],ecx
	__pop edi, ecx, ebx
	ret


;;;
;;; new_alphabetentry
;;; 
; In: cx-Prefix;dx-Tail
	align 32
new_alphabetentry:
	call check_bit_width
	;; eax is already [alphabet_pos]
	shl eax,2
	add eax,[alph]
	;; 	mov [eax+0],cx    ; Prefix
	;; 	mov [eax+2],dx    ; Tail
	shl edx,16
	mov dx,cx
	mov [eax],edx
	inc dword [alphabet_pos]
	ret


;;;
;;; search_entry
;;;
;;; cx=entry;edi=src_pos
	align 32
search_entry:
; Wenn Pos+Alphabeteintrag=Lnge_Quelle, dann 'raus
	mov eax,edi
	add eax,ebx
	cmp eax,[src_end]
	jb .j2
.j1:
	stc     ; not found
	ret
	align 32
.j2:
	mov eax,[alph]
	align 32
.j3:
	inc esi
	cmp esi,[alphabet_pos]
	jae .j1
; Gesuchter Alphabeteintrag?
	cmp cx,[edx+esi*4+0]
	jne .j3
	clc     ; found!
	ret

;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: