﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;

namespace VizBin
{
    class DefaultImageProcessor : AbstractImageProcessor
    {
        
        // the name of this image rendering engine
        private const String XS_NAME = "ASCII Plot";

		public override Bitmap createImage(byte[] vab_bytes, int vn_width, int vn_height)
		{
			return DefaultImageProcessor.createImage(vab_bytes, vn_width, vn_height, false);
		}

		public override Bitmap createOverviewImage(byte[] vab_bytes, int vn_width, int vn_height)
		{
			return DefaultImageProcessor.createImage(vab_bytes, vn_width, vn_height, true);
		}

        /// <summary>
        /// Given a byte array, and the dimensions of an image to be created, generate a bitmap
        /// (Image) with those dimenssions and generate the image. In our case, we will just color
        /// the respective pixels which represent the bits in the byte array.
        /// 
        /// Any classes which wish to be called from the controller must follow the same template
        /// as this class.
        /// </summary>
        /// <param name="vab_bytes">byte array</param>
        /// <param name="vn_width">width of the image</param>
        /// <param name="vn_height">height of the image</param>
        /// <param name="vb_is_overview">true if this is an imagemap false otherwise</param>
        /// <returns></returns>
        private static Bitmap createImage( byte[] vab_bytes, int vn_width, int vn_height, bool vb_is_overview  )
        {
            Bitmap o_image = null;
            if (vn_width > 65535 || vn_width < 1)
                throw new ArgumentException("Width cannot be greater than 65534 and less than 1");
            else if (vn_height > 65535 || vn_width < 1)
                throw new ArgumentException("Height cannot be greater than 65534 and less than 1");
            else if (vab_bytes == null || vab_bytes.Length < 1)
                throw new ArgumentException("byte array cannot be null or have a length of 0");
            else
            {
                o_image = new Bitmap(vn_width, vn_height);
                // vars needed for image creation
                int x = 0;
                int y = 0;
                Color o_curr_color = Color.White;

                // if the number of bytes given is less than we need to create a full size
                // image, perform a 'zoom' function by finding a rough estimate number of 
                // bytes required to fill the image and represent all of the bytes
                int n_zoom = AbstractImageProcessor.getScalingFactor(vn_width, vn_height, vab_bytes.Length) ;

                // the loop will be broken out of when we reach the end of the byte array
                foreach( byte buffer in vab_bytes )
                {

                    // determine what color to use
                    // tab character / space character
					if (buffer == 32 || buffer == 9)
						o_curr_color = Color.Red;
                    //CRLF
                    else if (buffer == 10 || buffer == 13)
                        o_curr_color = Color.Yellow;
                        // all printable ASCII
                    else if( buffer >= 33 && buffer <= 126 )
						o_curr_color = Color.Gray;
					else
						o_curr_color = Color.Gainsboro;

					// if this is an overview, draw based on byte values and don't draw lines of bits
					// if (vb_is_overview)
					//if( true )
					//{
                    n_zoom = 1;
                    for (int i = 0; i < n_zoom; i++)
                    {
                        // if we're at the end of the line, wrap to the next
                        if (x + 1 >= vn_width)
                        {
                            x = 0;
                            y++;
                        }
                        else
                        {
                            x++;
                        }
                        o_image.SetPixel(x, y, o_curr_color);
                    }
					//}
					//else
					//{
					//    // draw on a per pixel = bit basis
					//    for (int bit = 7; bit >= 0; bit--)
					//    {
					//        // determine what point to draw
					//        // if the byte was a 13 (CR), then start a new line
					//        if (/*(buffer == 13) ||*/ (x + 1 >= vn_width))
					//        {
					//            x = 0;
					//            y++;
					//        }
					//        else
					//            x++;
					//        o_image.SetPixel(x, y, o_curr_color);
					//    }//endfor
					//}
                }//endfor
            }//endif
            return o_image;
        }

        /// <summary>
        /// Returns the name of this image processor, in this case "Default Image Processor"
        /// </summary>
        /// <returns>the name of this image processor</returns>
        public override string getName()
        {
            return XS_NAME;
        }
    }
}
