﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Drawing;

namespace VizBin
{
    class LZM : AbstractImageProcessor
    {
        //string s = Convert.ToString(b, 16).PadLeft(2, '0');
        // the required name of this viz
        private const String XS_NAME = "Entropy";

        /// <summary>
        /// </summary>
        /// <seealso cref="AbstractImageProcessor.getName()"/>
        /// <returns></returns>
        public override string getName()
        {
            return XS_NAME;
        }

        /// <summary>
        /// </summary>
        /// <seealso cref="AbstractImageProcessor.getName()"/>
        /// <returns></returns>
        public override Bitmap createImage(byte[] vab_bytes, int vn_width, int vn_height)
        {
			return LZM.createImage(vab_bytes, vn_width, vn_height, false);
        }

        /// <summary>
        /// </summary>
        /// <seealso cref="AbstractImageProcessor.getName()"/>
        /// <returns></returns>
        public override Bitmap createOverviewImage(byte[] vab_bytes, int vn_width, int vn_height)
        {
			return LZM.createImage(vab_bytes, vn_width, vn_height, true);
        }

        /// <summary>
        /// Creates a detailed or overview image based on the value of the bool flag.
        /// </summary>
        /// <param name="vab_bytes"></param>
        /// <param name="vn_width"></param>
        /// <param name="vn_height"></param>
        /// <param name="vab_overview"></param>
        /// <returns></returns>
        private static Bitmap createImage(byte[] vab_bytes, int vn_width, int vn_height, bool vab_overview)
        {
			Bitmap o_image = new Bitmap(vn_width, vn_height);
            IDictionary<String, Int32> o_map = new Dictionary<String, Int32>();
            ICollection<String> o_entropy_stream = new LinkedList<String>();

            //prepopulate the map with single byte values
            foreach( Byte b in vab_bytes )
            {
                // get the key value
				String s_byte = Convert.ToString(b, 16).PadLeft(2, '0');
                // if the key is not in the map, insert it
                if (!o_map.ContainsKey(s_byte))
                    o_map[s_byte] = o_map.Count + 1;
            }

            // now run limpil ziv against the byte array
            int idx = 0;
			int n_largest_key_length = 0;
            while (idx < vab_bytes.Length)
            {
                byte b_curr = vab_bytes[idx];
				String s_key = Convert.ToString(b_curr, 16).PadLeft(2, '0');
				bool b_found = true;
                while (b_found)
                {
                    // break out of the loop so we can exit
                    if (idx + 1 >= vab_bytes.Length)
                        break;

					// the next byte in the array - we know we're not at the end yet
					// since we checked it in the above if statement
                    byte b_next = vab_bytes[idx + 1];
					String s_next_key = s_key + Convert.ToString(b_next, 16).PadLeft(2, '0');
                    b_found = o_map.ContainsKey(s_next_key);

					// if the key is not found, then add it to the map with the value of
					// map entry count + 1
					// otherwise, set the key to be the current next key and loop, update the idx
					// to point to the next byte
					if (!b_found)
					{
						o_map[s_next_key] = o_map.Keys.Count;
					}
					else
					{
						s_key = s_next_key;
						idx++;
					}
                }
				//o_entropy_stream.Add(o_map[s_key]);
				o_entropy_stream.Add(s_key);
				idx++;

				// keep the value of the longest key to be used in later calculations
				if (n_largest_key_length < s_key.Length)
					n_largest_key_length = s_key.Length;
            }//end while

			//generate the color map based on the keys in the map
			ICollection<Color> o_color_list = new LinkedList<Color>();
			
			// create the required color map
			int n_gradient = 255 / (n_largest_key_length / 2);
			for (int i = 0; i < (n_largest_key_length/2); i++)
				o_color_list.Add( Color.FromArgb(255 - (n_gradient * i), 0, (n_gradient * i)));

			// now that we have the LMZ entropy array, create the picture
			int x = 0;
			int y = 0;
			foreach(String s in o_entropy_stream )
			{
				Color o_paint = Color.Green;
				o_paint = o_color_list.ElementAt((s.Length/2) - 1);
				int n_paint_length = s.Length / 2;

				// paint the pixels
				// if we're at the end of the line, wrap to the next
				for (int i = 0; i < n_paint_length; i++)
				{
					if (x + 1 >= vn_width)
					{
						x = 0;
						y++;
					}
					else
						x++;
					o_image.SetPixel(x, y, o_paint);
				}
			}//end foreach

			//// if the generated image content is smaller than the height, resize the image
			//if (y < vn_height)
			//{
			//    y = (y < 1 ? 1 : y);
			//    if (y == 1 && x < vn_width)
			//    {
			//        x = (x < 1 ? 1 : x);
			//        o_image = new Bitmap(o_image, x, y);
			//    }
			//    else
			//        o_image = new Bitmap(o_image, vn_width, y);
			//}
			return o_image;
        }//end createImage
	}//endclass
}//endpackage
