﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VizBin.searcher
{
	abstract class AbstractSearch : Search
	{


        /// <summary>
        /// Returns a user friendly name of the type of search provided. This method
        /// should NEVER return a null or an empty string as it's output is used for
        /// UI rendering.
        /// </summary>
        /// <returns>String</returns>
        public abstract String getName();

		/// <summary>
		/// Finds the given byte array in the provided byte array and returns zero or more search result objects
		/// in the form if an IList collection. This method will never return null. Proxies all of the requests to the
		/// private static method contained within this class.
		/// </summary>
		/// <param name="vab_bytes_to_find">byte array to find</param>
		/// <param name="vs_byte_descriptor">Description of the byte array to find</param>
		/// <param name="vab_to_search">byte array to search</param>
		/// <returns>an IList of zero or more search results. If the array to be searched is smaller than the number
		/// of bytes provided to look for, an empty IList is returned. If either array is null, an exception is
		/// thrown. If the bytes to find is of length of 0, an empty List is returned.</returns>
		/// <exception cref="ArgumentException">thrown if either parameter is null</exception>
		public virtual ICollection<SearchResult> getResults(String vs_byte_descriptor, byte[] vab_bytes_to_find, byte[] vab_to_search)
		{
			return AbstractSearch.getResult(vs_byte_descriptor, vab_bytes_to_find, vab_to_search);
		}

		/// <summary>
		/// Finds the given byte array in the provided byte array and returns zero or more search result objects
		/// in the form if an IList collection. This method will never return null.
		/// </summary>
		/// <param name="vab_bytes_to_find">byte array to find</param>
		/// <param name="vs_byte_descriptor">Description of the byte array to find</param>
		/// <param name="vab_to_search">byte array to search</param>
		/// <returns>an IList of zero or more search results. If the array to be searched is smaller than the number
		/// of bytes provided to look for, an empty IList is returned. If either array is null, an exception is
		/// thrown. If the bytes to find is of length of 0, an empty List is returned.</returns>
		/// <exception cref="ArgumentException">thrown if either parameter is null</exception>
		private static ICollection<SearchResult> getResult(String vs_byte_descriptor, byte[] vab_bytes_to_find, byte[] vab_to_search)
		{
			if (vab_bytes_to_find == null)
				throw new ArgumentException("Bytes to find is null");
			if (vab_to_search == null)
				throw new ArgumentException("Bytes to search are null");

			ICollection<SearchResult> o_results = new LinkedList<SearchResult>();
			if (vab_bytes_to_find.Length > 0 || vab_bytes_to_find.Length <= vab_to_search.Length)
			{
				// stop when the bytes left to search are less than the bytes to find
				long l_remaining_to_search = vab_to_search.Length - vab_bytes_to_find.Length;

				// iterate through all of the bytes in the array and look for the first byte in the array of bytes to find
				for (long l_search_index = 0; l_search_index < l_remaining_to_search; l_search_index++)
				{
					// if the first byte matches the array we're looking for, create a temp and pass it to
					// our check method. if a true is returned, create a search result and add it to the list
					if (vab_bytes_to_find[0] == vab_to_search[l_search_index])
					{
						byte[] ab_temp = new byte[vab_bytes_to_find.Length];
						for (int i = 0; i < ab_temp.Length; i++)
							ab_temp[i] = vab_to_search[l_search_index + i];
						if (AbstractSearch.isEqual(vab_bytes_to_find, ab_temp))
							o_results.Add(new SearchResult(vs_byte_descriptor, l_search_index));
					}
				}

			}
			return o_results;
		}

		/// <summary>
		/// Return true if the byte array elements are exactly the same using the == method.
		/// </summary>
		/// <param name="vab_ref_bytes">the bytes we're looking for</param>
		/// <param name="vab_search_bytes">the bytes to check</param>
		/// <returns>true if they match, false otherwise</returns>
		private static bool isEqual(byte[] vab_ref_bytes, byte[] vab_search_bytes)
		{
			bool b_eq = vab_ref_bytes.Length == vab_search_bytes.Length;
			if (b_eq)
			{
				for (int i = 0; i < vab_ref_bytes.Length && b_eq == true; i++)
					b_eq = vab_ref_bytes[i] == vab_search_bytes[i];
			}
			return b_eq;
		}
	}
}
