﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VizBin.searcher
{
	/// <summary>
	/// Defines the interface that all classes which wish to be deemed capable of searching
	/// and included by the dynamic inclusion algorithm must implement.
	/// </summary>
    public interface Search
	{
		/// <summary>
		/// Returns the name of this search routine.
		/// </summary>
		/// <returns>String</returns>
		String getName();

        /// <summary>
        /// Finds all instances of the specified binary pattern in a file.
        /// 
        /// Given the byte array (one or more bytes, any less will throw an exception), find the number of instances of the
        /// search and return the results in a Collection of items which will contain 0 or more items.
        /// </summary>
		/// <param name="vs_byte_descriptor">Description of the byte array to find</param>
        /// <param name="vab_bytes_to_find">byte array of size 1 or more</param>
		/// <param name="vab_to_search">byte array of size 1 or more to search</param>
        /// <returns>IList of SearchResult objects</returns>
        ICollection<SearchResult> getResults(String vs_byte_descriptor, byte[] vab_bytes_to_find, byte[] vab_to_search);
	}
}
