﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VizBin.searcher
{
	/// <summary>
	/// Defines an object to be returned from a search. It has the results as a String object
	/// and the offset 
	/// </summary>
	public class SearchResult
	{
		private String is_result = null;
		private long il_offset = -1;

		/// <summary>
		/// Create a SearchResult object with the specified result as a string and 
		/// the offset as a long.
		/// </summary>
		/// <param name="vs_result">the search result as a string</param>
		/// <param name="vl_offset">the offset the result was found as a long</param>
		/// <exception cref="ArgumentException">if the string is null or has a length of 0, or the 
		/// offset value is less than 0</exception>
		internal SearchResult(String vs_result, long vl_offset)
		{
			if (vs_result == null || vs_result.Length < 1)
				throw new ArgumentException("Null Result or empty string is not allowed");
			if (vl_offset < 0)
				throw new ArgumentException("Offset must be greater or equal to 0");
			this.il_offset = vl_offset;
			this.is_result = vs_result;
		}

		/// <summary>
		/// Override the default ToString method to return a useful value
		/// </summary>
		/// <returns>string</returns>
		public override String ToString()
		{
			return this.is_result + " at: " + this.il_offset;
		}

		/// <summary>
		/// Return the result. This should never return null.
		/// </summary>
		/// <returns>string</returns>
		String getResult()
		{
			return this.is_result;
		}

		/// <summary>
		/// Return the offset, in the file, at which this specific result was found.
		/// </summary>
		/// <returns></returns>
		long getOffset()
		{
			return this.il_offset;
		}
	}
}
