﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using VizBin.searcher;

namespace VizBin.util
{
	/// <summary>
	/// Maps the enumeration (part of this class) onto byte arrays that represent the general categories represented
	/// by the enums.
	/// </summary>
	class MagicMap
	{
		// backing store map
		private IDictionary<MagicCategories, List<SearchResult>> io_backing_map = new Dictionary<MagicCategories, List<SearchResult>>();

		/// <summary>
		/// Enum categories representing the different types of things that can be searched for.
		/// </summary>
		public enum MagicCategories
		{
			IMAGES, PDF, OFFICE_DOCS, VIDEO, AUDIO, OTHER, ALL
		}

		/// <summary>
		/// Default constructor will prefill the backing Map &lt; Enum, IList &gt;
		/// </summary>
		public MagicMap()
		{
			foreach( MagicCategories mc in Enum.GetValues(typeof(MagicMap)))
				if( mc != MagicCategories.ALL )
					io_backing_map[mc] = new List<SearchResult>();
		}

		/// <summary>
		/// Sets the results for the given enum type. This will replace any previous results already
		/// set. Note that setting the ALL category will do nothing as it is a meta category.
		/// </summary>
		/// <param name="ve_category">the category of results to set</param>
		/// <param name="vo_results">the result list to use</param>
		/// <exception cref="ArgumentException">thrown if either param is null</exception>
		internal void setResults(MagicCategories ve_category, IList<SearchResult> vo_results)
		{
			if (vo_results == null)
				throw new ArgumentException("Result list cannot be null");
			if (ve_category != MagicCategories.ALL)
			{
				// do stuff here
			}
		}

		/// <summary>
		/// Get the results for a specific category. If the cagetory is "ALL" then all results will be returned.
		/// </summary>
		/// <param name="ve_category">MagicCategories enum</param>
		/// <returns>IList of SearchResult objects</returns>
		/// <exception cref="ArgumentException">thrown if the category is null</exception>
		internal IList<SearchResult> getResults( MagicCategories ve_category )
		{
			List<SearchResult> o_results = null;
			switch( ve_category )
			{
				case MagicCategories.ALL:
					o_results = new List<SearchResult>();
					foreach( MagicCategories mc in Enum.GetValues(typeof(MagicCategories)))
						if (mc != MagicCategories.ALL)
							o_results.AddRange(this.io_backing_map[ve_category]);
					break;
				default:
					o_results = new List<SearchResult>( io_backing_map[ve_category] );
					break;
			}
			return o_results;
		}
	}
}
